function PM_I_mean = summarize_replicate_PM(PM_I, repl_idx)

% PM_I_mean = summarize_replicate_PM(PM_I, [repl_idx])
%
% Computes mean intensities per probe across replicate arrays.
%
% PM_I -- matrix of perfect match probe intensities (#replicates x #PM probes)
% repl_idx -- optional index specifying replicates, e.g. [1 1 2 2] would
%   summarize the first to rows of PM_I into the first row of PM_I_mean
%   and rows 3 and 4 of PM_I into the second row of PM_I_mean
% returns mean intensities for each probe across replicates. NaN is only
%   returned as mean intensity if all replicates for this probe are NaN.
%
% written by Georg Zeller, MPI Tuebingen, Germany, 2007-2008

VERBOSE = 0;

if exist('repl_idx', 'var'),
  samples = unique(repl_idx);
else
  samples = 1;
  repl_idx = ones(1, size(PM_I, 1));
end
for s=1:length(samples),
  % take mean of replicates (ignoring NaN)
  idx = find(repl_idx == samples(s));
  PM_I_mean(samples(s),:) = mean(PM_I(idx,:));

  % correct probes for which at least one array has NaN
  nan_idx = find(isnan(PM_I_mean(samples(s),:)));
  PM_num_replic = nan(size(nan_idx));
  for i=1:length(nan_idx),
    values = PM_I(idx,nan_idx(i));
    values(isnan(values)) = [];
    PM_num_replic(i) = length(values);
    if isempty(values),
      PM_I_mean(samples(s),i) = nan;
    else
      PM_I_mean(samples(s),i) = mean(values);
    end
  end
  assert(~any(isnan(PM_num_replic)));
  assert(all(0<=PM_num_replic & PM_num_replic<=length(idx)));
  if VERBOSE,
    fprintf('  PM NaN in all %i arrrays:     %7i\n', length(idx), ...
            sum(PM_num_replic==0));
    for i=length(idx)-1:-1:1,
      fprintf('  PM NaN in %i of %i arrrays:    %7i\n', length(idx)-i, ...
              length(idx), sum(PM_num_replic==1));
    end
    fprintf('  PM NaN in at least 1 arrray: %7i\n', length(PM_num_replic));
  end
end

% eof