function [genes I_transcript dist]  = transcript_intensities_3p_bias(I, probe_id, probe_label, genes, LABEL)

% [genes, I_transcript, dist] = transcript_intensities_3p_bias(I, 
%                                probe_id, probe_label, genes, LABEL)
%
% Computes median intensities across all probes complementary to the same
% transcript along with information on the distance of probes to the 3'
% transcript end.
%
% I -- a vector of hybridization intensities of the same size as probe_id
% probe_id -- genomic locations (chr*10^8+position) of each probe
% probe_label -- a global annotation label for each probe (should be ==
%   LABEL.ambiguous in case of e.g. overlapping genes)
% genes -- a struct array with annotated genes
% LABEL -- a struct with all possible probe label constants
% returns a gene struct augmented by a hyb field (a struct with
%   gene-specific hybridization values) as well as a field with distance
%   for each probe to the 3' transcript end. also returns the ideal
%   transcript intensity for each probe as well as a global vector of
%   transcript distances
%
% written by Georg Zeller, MPI Tuebingen, Germany, 2007-2008

fprintf('Computing transcript intensity medians...\n');
I_transcript = -ones(size(probe_id));
dist         = zeros(size(I_transcript));
dist_counts  = zeros(size(I_transcript));

tmp = [genes.transcripts];
tmp = length(tmp);

transcript_medians = zeros(1,tmp);
gene_idx = zeros(1,tmp);

cnt = 1;

cover = zeros(size(I_transcript));

for g=1:length(genes),
  probe_idx = genes(g).probes(:,2);
  gene = genes(g);
  chr = str2num(gene.chr(4));
  genes(g).hyb.I = I(probe_idx)';
  genes(g).probes_3p_dist = -ones(size(gene.probes_exonic));
    
  for i=1:length(gene.transcripts)
    gene_idx(cnt) = g;
    
    exons      = gene.exons{i};
    exon_idx   = find(genes(g).probes_exonic(:,i)>0);
    intron_idx = find(genes(g).probes_intronic(:,i)>0);
    
    % determine dist to transcript end
    intron_str = ones(1,gene.stop-gene.start+1);
    switch gene.strands(i),
     case '+',
      gene_end = exons(end,2);
      probe_dist = gene_end - (gene.probes(exon_idx,1)-chr*100000000) + 1;
      assert(all(probe_dist>0));
      
      for e=1:size(exons,1)
        intron_str(gene_end-exons(e,2)+1:gene_end-exons(e,1)+1) = 0;
      end
     case '-',
      gene_end = exons(1,1);
      probe_dist = (gene.probes(exon_idx,1)-chr*100000000) - gene_end + 1;
      assert(all(probe_dist>0));
      for e=1:size(exons,1),
        intron_str(exons(e,1)-gene_end+1:exons(e,2)-gene_end+1) = 0;
      end
    end
    
    d = probe_dist;
    for p=1:length(probe_dist),
      assert(all(intron_str(d(p)-12:d(p)+12)==0));
      d(p) = d(p) - sum(intron_str(1:d(p)));
      assert(d(p)>0 & d(p)<=20000);
      genes(g).probes_3p_dist(exon_idx(p),i) = d(p);
    end
    
    % convert into global indices
    exon_idx   = probe_idx(exon_idx);
    intron_idx = probe_idx(intron_idx);
      
    % I_transcript in exons is the median of the exon probes
    I_tmp = I(exon_idx);
    I_tmp = I_tmp(~isnan(I_tmp));
    if ~isempty(I_tmp),
      cover(exon_idx) = cover(exon_idx) + 1;
      I_transcript(exon_idx) = median(I_tmp);
      genes(g).hyb.I_transcript(i) = median(I_tmp);
      transcript_medians(cnt) = median(I_tmp);
    else
      genes(g).hyb.I_transcript(i) = nan;
    end

    % I_transcript in introns is the median of the intron probes
    I_tmp = I(intron_idx);
    I_tmp = I_tmp(~isnan(I_tmp));
    if ~isempty(I_tmp),
      cover(intron_idx) = cover(intron_idx) + 1;
      I_transcript(intron_idx) = median(I_tmp);
    end
    
    if all(dist_counts(exon_idx) == 0),
      dist(exon_idx) = d;
      dist_counts(exon_idx) = dist_counts(exon_idx) + 1;
    else % average over all genes at one position
      for e=1:length(exon_idx),
        dist(exon_idx(e)) = (dist(exon_idx(e)) ...
                             * dist_counts(exon_idx(e)) ...
                             + d(e)) ...
            / (dist_counts(exon_idx(e)) + 1);
        dist_counts(exon_idx(e)) = dist_counts(exon_idx(e)) + 1;
      end
    end
    
    cnt = cnt + 1;
  end
  assert(size(genes(g).probes_3p_dist,1) == size(genes(g).probes,1));
%  if mod(g,100)==0, fprintf('  %i\r',g); end
end
%fprintf('\n');

% reset I_transcript at sites that are labeled ambiguous
I_transcript(probe_label==LABEL.ambiguous) = -1;

% I_transcript in intergenic regions is global intergenic median
global_intergenic_idx = find(probe_label==LABEL.intergenic);
I_tmp = I(global_intergenic_idx);
I_transcript(global_intergenic_idx) = median(I_tmp(~isnan(I_tmp)));

I_transcript(find(cover>1)) = -1;
fprintf('%2.1f%% of probes masked due to coverage > 1.\n', ...
        100*mean(cover>1));

