function prepare_data(data_dir, fn_arrays, fn_out)

% prepare_data(data_dir, fn_arrays, fn_out)
%
% Loads and converts array data into a format useful for segmentation
% with mSTAD.
%
% data_dir -- the path where the array data is stored
% fn_arrays -- a cell array of file names to be prepared for
%   segmentation
% fn_out -- name prefix of the files to which the output is written;
%   there will be a file containing only the information necessary for
%   segmentation (with _segm extension) and one with more annotations
%   (with _segm_annot extension) 
% 
% written by Georg Zeller, MPI Tuebingen, Germany, 2007-2008

% adjust directories in set_mSTAD_paths.m to make sure that all includes work
addpath ../
set_mSTAD_paths

% number of discrete gene expression levels
NUM_EXPR_LEVELS = 10;
% version of the TAIR gene annotation
ANNOTATION_TAG  = 'TAIR7';

% directory where mappings between array coordinates, genome and genes are stored
map_dir = '../../../PSB_data/maps/';
% directory to which output is written
out_dir = '../../../PSB_data/mSTAD/input_data/';

% file with mapping between array coordinates and genome location 
fn_genomic_probes = [map_dir 'genomic_probes'];
% file with mapping between array coordinates and genes
fn_genic_probes   = [map_dir 'genic_probes_' ANNOTATION_TAG];


% get mapping probes -> genome
fprintf('Loading maping probes -> genome...\n\n');
load(fn_genomic_probes, 'probe_id', ...
     'probe_PM_array_idx');  

% get mapping probes -> genes
fprintf('Loading mapping probes <-> genes...\n\n');
load(fn_genic_probes, 'probe_label', 'genes');  


% get intensities
probe_PM_I = zeros(length(fn_arrays),length(probe_id));
fprintf('Loading array intensities...\n\n');
for f=1:length(fn_arrays),
  tmp = load([data_dir fn_arrays{f}], 'array');
  arrays(f) = tmp.array;
end
clear tmp

% average over replicates
for p=1:length(probe_PM_array_idx),
  for f=1:length(fn_arrays),
    probe_PM_I(f,p) = arrays(f).mean(probe_PM_array_idx(p));
  end
  if mod(p,10000)==0, fprintf('  %i\r',p); end
end
fprintf('\n');

% log_2 transform intensities
PM_I = log2(probe_PM_I);
if length(fn_arrays)>1,
  fprintf('summarizing intensities across replicates...\n');
  PM_I = summarize_replicate_PM(PM_I);
end
clear probe_PM_I probe_PM_array_idx

% map between genes and probes
LABEL = get_label_set_mSTAD();
genes = median_transcript_intensities(PM_I, probe_id, probe_label, ...
                                      genes, LABEL);

fprintf(' exonic probes:     %2.1f%% (mean I: %2.2f)\n', ...
        100*mean(probe_label==LABEL.exonic), ...
        mean(PM_I(probe_label==LABEL.exonic & ~isnan(PM_I))));
fprintf(' intronic probes:   %2.1f%% (mean I: %2.2f)\n', ...
        100*mean(probe_label==LABEL.intronic), ...
        mean(PM_I(probe_label==LABEL.intronic & ~isnan(PM_I))));
fprintf(' intergenic probes: %2.1f%% (mean I: %2.2f)\n', ...
        100*mean(probe_label==LABEL.intergenic), ...
        mean(PM_I(probe_label==LABEL.intergenic & ~isnan(PM_I))));

fprintf('\nMapping genes to probes...\n');
gene_id          = cell(size(probe_label));
gene_num         = zeros(size(probe_label));
gene_orientation = cell(size(probe_label));
for g=1:length(genes),
  assert(all(genes(g).strands==genes(g).strands(1)));
  probe_idx = genes(g).probes(:,2);
  if ~isempty(probe_idx),
    assert(~any(probe_label(probe_idx)==LABEL.intergenic));
    gene_num(probe_idx) = gene_num(probe_idx) + 1;
    for i=1:length(probe_idx),
      p = probe_idx(i);
      gene_id{p}(end+1) = g;
      switch genes(g).strands(1),
        case '+',
         gene_orientation{p}(end+1) = +1;
        case '-',
         gene_orientation{p}(end+1) = -1;
      end
    end
  end
  if mod(g,100)==0, fprintf('  %2.2f%%\r', 100*g/length(genes)); end
end
fprintf('  %2.2f%%\n', 100*g/length(genes));


% partition chromosome data into blocks around genes
fprintf('\nPartitioning data\n');
intergenic_blocks = find_blocks(probe_label==LABEL.intergenic);

upstr_gene   = zeros(size(probe_label));
downstr_gene = zeros(size(probe_label));
upstr_dist   = inf(size(probe_label));
downstr_dist = inf(size(probe_label));

probe_chr = floor(probe_id/100000000);
gene_chr = [genes.chr];
gene_chr = str2num(gene_chr(4:4:4*length(genes))')';
assert(length(gene_chr) == length(genes));
for b=1:size(intergenic_blocks,2),
  ige_idx = intergenic_blocks(1,b):intergenic_blocks(2,b);
  if b>1,
    g = [gene_id{intergenic_blocks(1,b)-1}];
    assert(all(g~=0));
    g_chr = unique(gene_chr(g));
    assert(length(g_chr)==1);
    % as gene end take the end of the last gene
    [gene_end idx] = max([genes(g).stop] ...
        + probe_chr(intergenic_blocks(1,b)-1)*100000000);
    g = g(idx);
    d = probe_id(ige_idx) - gene_end;
    d(d<0) = inf;
    d(probe_chr(ige_idx)~=g_chr) = inf;
    [upstr_dist(ige_idx) idx] = min([upstr_dist(ige_idx); d]);
    upstr_gene(ige_idx(idx==2)) = g;
  end
  
  if b<size(intergenic_blocks,2),
    g = [gene_id{intergenic_blocks(2,b)+1}];
    assert(all(g~=0));
    g_chr = unique(gene_chr(g));
    assert(length(g_chr)==1);
    % as gene start take the start of the first gene
    [gene_start idx] = min([genes(g).start] ...
        + probe_chr(intergenic_blocks(2,b)+1)*100000000);
    g = g(idx);
    d = gene_start - probe_id(ige_idx);
    d(d<0) = inf;
    d(probe_chr(ige_idx)~=g_chr) = inf;
    [downstr_dist(ige_idx) idx] = min([downstr_dist(ige_idx); d]);
    downstr_gene(ige_idx(idx==2)) = g;
  end
  fprintf('  %2.2f%%\r', b, 100*b/size(intergenic_blocks,2));
end
fprintf('  %2.2f%%\n', b, 100*b/size(intergenic_blocks,2));

% subdivide into chunks of ~1MB 
chunk_size = 1000000;
chunk_id = make_chunks(probe_id, probe_chr, intergenic_blocks, chunk_size);
assert(~any(chunk_id==0));

% assign each chunk to one of 5 subsets
chunks = unique(chunk_id);
num_subsets = 5;
num_chunks = length(chunks);
chunks = randperm(num_chunks);
chunk_subsets = cell(num_subsets);
for i=1:num_subsets-1,
  idx = 1:round(num_chunks/num_subsets);
  chunk_subsets{i} = chunks(idx);
  chunks(idx) = [];
  fprintf('  size of subset %i: %i chunks, %i probes.\n', i, ...
          length(chunk_subsets{i}), sum(ismember(chunk_id, chunk_subsets{i})));

  if i>1,
    assert(isempty(intersect(chunk_subsets{i}, chunk_subsets{i-1})));
  end
end
chunk_subsets{num_subsets} = chunks;
assert(isempty(intersect(chunk_subsets{num_subsets}, ...
                         chunk_subsets{num_subsets-1})));
fprintf('  size of subset %i: %i chunks, %i probes.\n', num_subsets, ...
        length(chunk_subsets{num_subsets}), ...
        sum(ismember(chunk_id, chunk_subsets{num_subsets})));

% split chunks in intergenic regions such that each resulting gene_block
% only contains a single genes (or several, but overlapping ones)
[gene_block_id, train_genes] = make_gene_blocks(probe_id, probe_chr, gene_id, ...
                                 intergenic_blocks);
fprintf('\n  partitioned data into %i single-gene blocks.\n', ...
        length(unique(gene_block_id)));


fprintf('\nFor unmasked genes:\n');
fprintf(' exonic probes:     %2.1f%% (mean I: %2.2f)\n', ...
        100*mean(probe_label==LABEL.exonic & train_genes), ...
        mean(PM_I(probe_label==LABEL.exonic & train_genes ...
                  & ~isnan(PM_I))));
fprintf(' intronic probes:   %2.1f%% (mean I: %2.2f)\n', ...
        100*mean(probe_label==LABEL.intronic & train_genes), ...
        mean(PM_I(probe_label==LABEL.intronic & train_genes ...
                  & ~isnan(PM_I))));
fprintf(' intergenic probes: %2.1f%% (mean I: %2.2f)\n', ...
        100*mean(probe_label==LABEL.intergenic & train_genes), ...
        mean(PM_I(probe_label==LABEL.intergenic & train_genes ...
                  & ~isnan(PM_I))));


% save extensive annotation
fn = [out_dir fn_out '_segm_annot'];
fprintf('\nSaving data to %s...\n', fn);
save(fn, 'PM_I', 'probe_id', 'probe_label', 'gene_id', 'gene_num', ...
     'gene_orientation', 'train_genes', 'upstr_gene','downstr_gene', ...
     'upstr_dist', 'downstr_dist', 'chunk_id', 'chunk_subsets', ...
     'gene_block_id', 'LABEL', 'NUM_EXPR_LEVELS', 'ANNOTATION_TAG');

% save only what is necessary for segmentation
signal = PM_I;
label = probe_label;
exm_id = gene_block_id;
fn = [out_dir fn_out '_segm'];
fprintf('\nSaving data to %s...\n', fn);
save(fn, 'signal', 'label', 'exm_id', 'probe_id', 'train_genes', 'chunk_id', ...
     'NUM_EXPR_LEVELS');

% eof
