% Main script for performance evaluation of mSTAD's predictions against
% annotated genes.
%
% written by Georg Zeller & Gunnar Raetsch, MPI Tuebingen, Germany, 2007-2008

% adjust set_mSTAD_paths.m to point to the correct directories
addpath ../
mSTAD_home = set_mSTAD_paths();
set_hmsvm_paths();

% basic data directory
base_dir = '../../../PSB_data/';
% file name from which training data will be loaded
data_dir = [base_dir 'mSTAD/input_data/'];
data_file = [data_dir 'T_003_ABC_R_A_QN_RR_segm'];

% name of the learning model
model_name = 'mSTAD_model';
% directory of the model-specific code
model_dir = sprintf('%s/%s', mSTAD_home, model_name);
if exist(model_name) ~= 7,
  addpath(model_dir);
end

% directories where the trained instances of mSTAD were stored (see
% out_dir in setup_training.m)
% adjust these to evaluate your own trained mSTAD models
pred_dirs = {[base_dir 'mSTAD/training/mSTAD_result_2008-09-18_17h37_model1/'], ...
             [base_dir 'mSTAD/training/mSTAD_result_2008-09-18_17h37_model2/'], ...
            };

% switch to evaluate on test examples (if 1) 
% or on validation examples (if 0)
USE_TEST_DATA = 0

% control of the amount of output
VERBOSE = 0

% maximum number of examples evaluated (just to save time)
% set this to inf, if you want to evaluate all available examples
MAX_NUM_EXM = 500;

% number of iterations used for training the mSTAD model 
% (if empty, the model from the final iteration will be loaded)
iteration = [];

model_config = model_config();
LABELS = get_label_set_mSTAD();

% seed for random number generation
rand('seed', 11081979);

comb_eval = [];
% evaluate individual predictors
for f=1:length(pred_dirs),
  if isempty(iteration),
    fn = sprintf('%slsl_final.mat', pred_dirs{f});
  else
    fn = sprintf('%slsl_iter%i.mat', pred_dirs{f}, iteration);
  end
  load(fn, 'PAR', 'score_plifs', 'transition_scores', 'state_model');
  fprintf('\nEvaluating %s...\n\n', fn);
  disp(PAR);
  assert(isequal(PAR.model_name, model_name));
%  assert(isequal(PAR.model_dir, model_dir));

  load(PAR.data_file, 'exm_id', 'signal', 'label', 'probe_id');  

  if USE_TEST_DATA,
    eval_exm_ids = PAR.test_exms;
  else
    eval_exm_ids = PAR.vald_exms;
  end
  % permute and subselect examples for evaluation
  eval_exm_ids = eval_exm_ids(randperm(length(eval_exm_ids)));
  eval_exm_ids = eval_exm_ids(1:min(MAX_NUM_EXM, length(eval_exm_ids)));
  assert(isempty(intersect(PAR.train_exms, eval_exm_ids)));
  if USE_TEST_DATA,
    fprintf('using %i genic regions from test set for evaluation.\n', length(eval_exm_ids));  
  else
    fprintf('using %i genic regions from validation set for evaluation.\n', length(eval_exm_ids));  
  end
  
  if VERBOSE>=1
    if ~exist('state_model', 'var'),
      state_model = eval(sprintf('%s(PAR, transition_scores);', model_config.func_make_model));
    end
    fhs = eval(sprintf('%s(state_model, score_plifs, PAR, transition_scores);', ...
                       model_config.func_view_model));
    assert(isequal(model_name, 'mSTAD_model'));
    print(fhs(1), '-depsc', '-r300', [pred_dirs{f} 'intergenic_plif.eps']);
    print(fhs(2), '-depsc', '-r300', [pred_dirs{f} 'exon_f_plifs.eps']);
    print(fhs(3), '-depsc', '-r300', [pred_dirs{f} 'exon_s_plifs.eps']);
    print(fhs(4), '-depsc', '-r300', [pred_dirs{f} 'intron_f_plifs.eps']);
    print(fhs(5), '-depsc', '-r300', [pred_dirs{f} 'intron_s_plifs.eps']);
    print(fhs(6), '-depsc', '-r300', [pred_dirs{f} 'transition_scores.eps']);
  end

  % determine globally optimal threshold on gene subset
  eval_idx = find(ismember(exm_id, eval_exm_ids));
  [tmp, global_thresh] = opt_threshold(signal(1,eval_idx), label(eval_idx));
  acc_global_thresh = zeros(1,length(eval_exm_ids)); 
  acc_local_thresh  = zeros(1,length(eval_exm_ids)); 
  
  for i=1:length(eval_exm_ids),
    idx = find(exm_id==eval_exm_ids(i));
    obs_seq = signal(:,idx);
    assert(~any(any(isnan(obs_seq))));
    true_label_seq = label(idx);
    % decode predicted label sequence
    pred_path = decode_Viterbi(obs_seq, transition_scores, score_plifs, PAR);
    pred_label_seq = pred_path.label_seq;
    % determine probe level accuracy
    good_pos = find(true_label_seq~=LABELS.ambiguous ...
                    & true_label_seq~=LABELS.genic);
    acc_LSL(i) = mean((true_label_seq(good_pos)==LABELS.exonic) == ...
                      (pred_label_seq(good_pos)==LABELS.exonic) ...
                      & (true_label_seq(good_pos)~=LABELS.exonic) == ...
                      (pred_label_seq(good_pos)~=LABELS.exonic));
    % evaluate accuracy of segmentation
    ev = evaluate_segmentation(true_label_seq, pred_label_seq, LABELS);
    % determine gene expression level
    median_exo_I = median(obs_seq(1, true_label_seq==LABELS.exonic));
    expr_level = find(PAR.expression_bins(:,1) <= median_exo_I ...
                      & median_exo_I < PAR.expression_bins(:,2)); 
    if isempty(expr_level),
      expr_level = nan;
    end
    assert(isnan(expr_level) || (expr_level>=1 && expr_level<=PAR.num_levels));
    ev.expr_level = expr_level;
    ev.method = 'mSTAD';
    if i==1,
      eval = ev;
    else 
      eval(i) = ev;
    end

    % threshold intensities of each example with global threshold
    acc_global_thresh(i) = opt_threshold(obs_seq(1,good_pos), ...
                                          true_label_seq(good_pos), ...
                                          global_thresh);
    % threshold intensities of each example with local threshold
    acc_local_thresh(i) = opt_threshold(obs_seq(1,good_pos), ...
                                         true_label_seq(good_pos));
    
    if VERBOSE>=2,
      print_evaluation(ev);
      view_label_seqs(figure(1), obs_seq, true_label_seq, pred_label_seq);
      pause
    end
  end
  comb_eval = [comb_eval eval];
  assert(all(acc_local_thresh>=acc_global_thresh));

  
  fprintf('\n LSL accuracy:                 %2.2f%%\n', 100*mean(acc_LSL));
  fprintf(' global thresholding accuracy: %2.2f%%\n', 100*mean(acc_global_thresh));
  fprintf(' local thresholding accuracy:  %2.2f%%\n', 100*mean(acc_local_thresh));
  
  % display evaluation result per predictor individually
  fh = print_evaluation(eval);
  if ~isempty(fh),
    print(fh, '-depsc', '-r300', sprintf('%sevaluation.eps', pred_dirs{f}));
  end
end

% display evaluation result combined for all predictors
fh = print_evaluation(comb_eval);
if ~isempty(fh),
  print_fn = pred_dirs{1}(1:strfind(pred_dirs{1}, '_model')-1);
  print_fn = [print_fn '_evaluation.eps'];
  print(fh, '-depsc', '-r300', print_fn);
end

% eof