function eval = evaluate_segmentation(true_label_seq, pred_label_seq, LABELS)

% eval = evaluate_segmentation(true_label_seq, pred_label_seq, LABELS)
%
% Evaluates the predicted segmentation of an examples sequence assessing
% sensitivity and specificity on the level of individual probes and exons,
% also determines the correctness of exon boundaries.
%
% true_label_seq -- label sequence representing the ground truth (the
%   gene annotation)
% pred_label_seq -- predicted label sequence
% LABELS -- a struct with fields specifying probe annotations
% returns a struct with fields containing performace estimates
%
% written by Georg Zeller, MPI Tuebingen, Germany, 2007-2008

% assess performance only on informative probes (ignoring ones that
% overlap intron/exon or exon/intergenic boundaries and are thus labeled
% 'ambiguous' or 'genic')
good_idx = find(true_label_seq==LABELS.exonic ...
                | true_label_seq==LABELS.intronic ...
                | true_label_seq==LABELS.intergenic);
true_label_seq = true_label_seq(good_idx);
pred_label_seq = pred_label_seq(good_idx);

eval.probe_ACC = mean((true_label_seq==LABELS.exonic) == ...
                      (pred_label_seq==LABELS.exonic) ...
                      & (true_label_seq~=LABELS.exonic) == ...
                      (pred_label_seq~=LABELS.exonic));
    
eval.probe_SN = sum(true_label_seq==LABELS.exonic ...
                    & pred_label_seq==LABELS.exonic) ...
                / sum(true_label_seq==LABELS.exonic);
    
eval.probe_SP = sum(true_label_seq==LABELS.exonic ...
                    & pred_label_seq==LABELS.exonic) ...
                / sum(pred_label_seq==LABELS.exonic);
    
true_exons = find_blocks(true_label_seq==LABELS.exonic);
pred_exons = find_blocks(pred_label_seq==LABELS.exonic);

% assess sensitivity
eval.exon_SN = 0;
eval.exon_FN = 0;
for e=1:size(true_exons,2),
  if all(pred_label_seq(true_exons(1,e):true_exons(2,e))==LABELS.exonic),
    eval.exon_SN = eval.exon_SN + 1;
  end

  if ~any(pred_label_seq(true_exons(1,e):true_exons(2,e))==LABELS.exonic),
    eval.exon_FN = eval.exon_FN + 1;
  end
end
eval.exon_SN = eval.exon_SN / size(true_exons,2);
eval.exon_FN = eval.exon_FN / size(true_exons,2);
     
% assess specificity
eval.exon_SP = 0;
eval.exon_FP = 0;
for e=1:size(pred_exons,2),
  if all(true_label_seq(pred_exons(1,e):pred_exons(2,e))==LABELS.exonic),
    eval.exon_SP = eval.exon_SP + 1;
  end
  
  if ~any(true_label_seq(pred_exons(1,e):pred_exons(2,e))==LABELS.exonic),
    eval.exon_FP = eval.exon_FP + 1;
  end
end
eval.exon_SP = eval.exon_SP / size(pred_exons,2);
eval.exon_FP = eval.exon_FP / size(pred_exons,2);

% assess exon boundary accuracy
exon_boundary_correct = size(intersect(true_exons', pred_exons', 'rows')',2);
eval.exon_boundary_SN = exon_boundary_correct / size(true_exons,2);
eval.exon_boundary_SP = exon_boundary_correct / size(pred_exons,2);

%exon_boundary_correct
%size(true_exons,2)
%size(pred_exons,2)
%keyboard

% replace NaN values caused by 0-denominators (in case of exon-free
% sequences)
if sum(true_label_seq==LABELS.exonic)==0,
  eval.probe_SN = nan;
end

if sum(pred_label_seq==LABELS.exonic)==0,
  eval.probe_SP = nan;
end

if size(true_exons,2)==0,
  eval.exon_SN = nan;
  eval.exon_FN = nan;
  eval.exon_boundary_SN = nan;
end

if size(pred_exons,2)==0,
  eval.exon_SP = nan;
  eval.exon_FP = nan;
  eval.exon_boundary_SP = nan;
end

% eof

