function loss_matrix = calc_loss_matrix(true_state_seq, state_model, PAR)

% loss_matrix = calc_loss_matrix(true_state_seq, state_model)
%
% Computes a loss matrix |S| x n where S is the set of states 
% and n the length of the true state sequence.
%
% true_state_seq -- true sequence of states (of length n) to be learned.
% state_model -- a struct specifying the state transition model (see
%   make_model_mSTAD.m for details)
% PAR -- a struct of parameters specified in setup_training.m and
%   train_hmsvm.m 
% returns the loss matrix used for decoding to obtain the max margin
%   violator
%
% written by Georg Zeller & Gunnar Raetsch, MPI Tuebingen, Germany, 2007-2008

fp_exon_loss           = 1;
fn_exon_loss           = 1;
intron_intergenic_loss = 0.1;
exon_level_loss        = 0.01;

STATES = get_state_set_mSTAD(PAR);

fn = fieldnames(STATES);
exo_f_idx = strmatch('exo_f', fn);
assert(length(exo_f_idx) == PAR.num_levels);
exo_s_idx = strmatch('exo_s', fn);
assert(length(exo_s_idx) == PAR.num_levels);
exo_idx = strmatch('exo', fn);
assert(length(exo_idx) == 2*PAR.num_levels);

ino_f_idx = strmatch('ino_f', fn);
assert(length(ino_f_idx) == PAR.num_levels);
ino_s_idx = strmatch('ino_s', fn);
assert(length(ino_s_idx) == PAR.num_levels);
ino_idx = strmatch('ino', fn);
assert(length(ino_idx) == 2*PAR.num_levels);

loss = zeros(length(state_model));

% Have a loss for confused exon states
loss([STATES.ige ino_idx'], exo_idx') = fn_exon_loss;
loss(exo_idx', [STATES.ige ino_idx']) = fp_exon_loss;

% Have a smaller loss for intron states confused with intergenic states
loss(STATES.ige, ino_idx') = intron_intergenic_loss;
loss(ino_idx', STATES.ige) = intron_intergenic_loss;

% Also penalize if expression levels are confused for exon states
% in a manner increasing with level difference
level_lm = zeros(PAR.num_levels);
for i=1:PAR.num_levels,
  level_lm = level_lm + diag(repmat(i*exon_level_loss,1,PAR.num_levels-i), i);
  level_lm = level_lm + diag(repmat(i*exon_level_loss,1,PAR.num_levels-i), -i);
end
loss(exo_f_idx, exo_f_idx) = level_lm;
loss(exo_s_idx, exo_s_idx) = level_lm;
loss(ino_f_idx, ino_f_idx) = level_lm;
loss(ino_s_idx, ino_s_idx) = level_lm;

%imagesc(loss);
%keyboard

for i=1:length(true_state_seq),
  loss_matrix(:,i) = loss(:,true_state_seq(i));
end

% eof