function [score_plifs transition_scores] = init_parameters(signal, label, state_model, PAR)

% [score_plifs transition_scores] = init_parameters(signal, label, state_model, PAR)
%
% Initializes the feature scoring PLiFs and transition scores.
%
% signal -- the feature matrix (sequence of observations) of size m x n
%   where m is equal to the number of features and n the combined length
%   of the training sequences
% label -- (combined) label sequence(s) of total length n
% state_model -- graphical model (see make_model_mSTAD.m)
% PAR -- a struct of parameters specified in setup_training.m and
%   train_hmsvm.m
% returns a struct representation of feature scoring functions
%   (score_plifs) and a vector of transition scores
%
% written by Georg Zeller & Gunnar Raetsch, MPI Tuebingen, Germany, 2007-2008

LABELS = get_label_set_mSTAD();
STATES = get_state_set_mSTAD(PAR);

% init a score PLiF for each combination of features and states
num_features = size(signal, 1);
for f=1:num_features,
  s = signal(f,:);
  a = signal(f,label==LABELS.exonic);
  s = sort(s);
  a = sort(a);

  % determine x values for supporting points of
  % intronic/intergenic PLiFs
  general_limits = linspace(1, length(s), PAR.num_plif_nodes-1);
  general_limits = ...
      round((general_limits(1:end-1)+general_limits(2:end))/2);
  general_limits = s([1 general_limits length(s)]);
  assert(length(general_limits) == PAR.num_plif_nodes);
  
  % determine x values for supporting points of exon PLiFs
  active_limits  = linspace(1, length(a), PAR.num_plif_nodes-1);
  active_limits = ...
      round((active_limits(1:end-1)+active_limits(2:end))/2);
  active_limits = a([1 active_limits length(a)]);
  assert(length(active_limits) == PAR.num_plif_nodes);

  % set default PLiF for all states 
  for s=1:length(state_model),
    score_plifs(f,s).limits = general_limits;
    score_plifs(f,s).scores = zeros(size(general_limits));
    score_plifs(f,s).dim = (f-1)*num_features + s;
  end

  % overwrite supporting points of exon PLiFs
  fn = fieldnames(STATES);
  exon_states = strmatch('exo_', fn);
  for i=1:length(exon_states),
    s = exon_states(i);
    score_plifs(f,s).limits = active_limits;
    score_plifs(f,s).penalties = zeros(size(active_limits));
  end
end

% init scores for transitions specified to have a score in make_model
for i=1:length(state_model),
  assert(length(state_model(i).successors) ...
         == length(state_model(i).trans_scores));
  idx = state_model(i).trans_scores;
  idx(idx==0) = [];
  transition_scores(idx) = 0;
end
transition_scores = transition_scores';

%view_model(state_model, score_plifs, transition_scores);
%keyboard

% eof