function state_seq = labels_to_states(label_seq, state_model, signal, PAR)

% state_seq = labels_to_states(label_seq, state_model, signal, PAR)
%
% Converts a label sequence into a state sequence.
%
% label_seq -- a sequence of labels (see get_label_set_mSTAD.m)
% state_model -- graphical model (see make_model_mSTAD.m)
% signal -- the feature matrix (sequence of observations) of size m x n
%   where m is equal to the number of features and n the combined length
%   of the training sequences
% PAR -- a struct of parameters specified in setup_training.m and
%   train_hmsvm.m
% returns the sequence of states corresponding to the given label
%   sequence
%
% written by Georg Zeller, MPI Tuebingen, Germany, 2007-2008

LABELS = get_label_set_mSTAD();
STATES = get_state_set_mSTAD(PAR);

state_seq = nan(size(label_seq));
state_seq(label_seq==LABELS.intergenic) = STATES.ige;

% determine the expression level in order to assign states of
% the corresponding discrete expression level
exo_idx = find(label_seq==LABELS.exonic);
median_exo_I = median(signal(1,exo_idx));
expr_level = find(PAR.expression_bins(:,1) <= median_exo_I ...
                  & median_exo_I < PAR.expression_bins(:,2)); 

exo_f_state = getfield(STATES, sprintf('exo_f_%02i', expr_level));
exo_s_state = getfield(STATES, sprintf('exo_s_%02i', expr_level));
ino_f_state = getfield(STATES, sprintf('ino_f_%02i', expr_level));
ino_s_state = getfield(STATES, sprintf('ino_s_%02i', expr_level));

state_seq(label_seq==LABELS.exonic)   = exo_f_state;
state_seq(label_seq==LABELS.intronic) = ino_f_state;

% intensity thresholds used to resolve ambiguous states
ige_exo_threshold = 0.5 * (mean(signal(1,label_seq==LABELS.exonic)) ...
                           + mean(signal(1,label_seq==LABELS.intergenic)));
ino_exo_threshold = 0.5 * (mean(signal(1,label_seq==LABELS.exonic)) ...
                           + mean(signal(1,label_seq==LABELS.intronic)));

unamb_state_seq = state_seq;
% resolve ambiguities
amb_blocks = find_blocks(isnan(state_seq));
for i=1:size(amb_blocks,2),
  l = amb_blocks(1,i)-1;
  r = amb_blocks(2,i)+1;
  if l < 1,
    state_seq(l+1:r) = state_seq(r);
  elseif r > length(label_seq),
    state_seq(l:r-1) = state_seq(l);
  elseif label_seq(l) == LABELS.intergenic && label_seq(r) == LABELS.exonic,
    for j=r-1:-1:l+1,
      if signal(1,j) > ige_exo_threshold,
        state_seq(j) = exo_f_state;
      else
        state_seq(l+1:j) = STATES.ige;
        break
      end
    end
  elseif label_seq(l) == LABELS.exonic && label_seq(r) == LABELS.intergenic,
    for j=l+1:1:r-1,
      if signal(1,j) > ige_exo_threshold,
        state_seq(j) = exo_f_state;
      else
        state_seq(j:r-1) = STATES.ige;
        break
      end
    end
  elseif label_seq(l) == LABELS.exonic && label_seq(r) == LABELS.intronic,
    for j=l+1:1:r-1,
      if signal(1,j) > ino_exo_threshold,
        state_seq(j) = exo_f_state;
      else
        state_seq(j:r-1) = ino_f_state;
        break
      end
    end
  elseif label_seq(l) == LABELS.intronic && label_seq(r) == LABELS.exonic,
    for j=r-1:-1:l+1,
      if signal(1,j) > ino_exo_threshold,
        state_seq(j) = exo_f_state;
      else
        state_seq(l+1:j) = ino_f_state;
        break
      end
    end
  elseif label_seq(l) == LABELS.exonic && label_seq(r) == LABELS.exonic,
    state_seq(l:r) = exo_f_state;
  elseif label_seq(l) == LABELS.intronic && label_seq(r) == LABELS.intronic,
    state_seq(l+1:r) = ino_f_state;
  elseif label_seq(l) == LABELS.intronic && label_seq(r) == LABELS.intergenic,
    % this can only be the case if the last exon ist only partially
    % covered by tiling probes. Assign an exon state to the ambigously
    % labeled probe with highest intensity
    [tmp, mx_idx] = max(signal(1,l+1:r-1));
    state_seq(l+mx_idx) = exo_f_state;
    for j=l+mx_idx-1:-1:l+1,
      if signal(1,j) > ino_exo_threshold,
        state_seq(j) = exo_f_state;
      else
        state_seq(l+1:j) = ino_f_state;
        break
      end
    end
    for j=l+mx_idx+1:1:r-1,
      if signal(1,j) > ige_exo_threshold,
        state_seq(j) = exo_f_state;
      else
        state_seq(j:r-1) = STATES.ige;
        break
      end
    end
  elseif label_seq(l) == LABELS.intergenic && label_seq(r) == LABELS.intronic,
    % this can only be the case if the first exon ist only partially
    % covered by tiling probes. Assign an exon state to the ambigously
    % labeled probe with highest intensity
    [tmp, mx_idx] = max(signal(1,l+1:r-1));
    state_seq(l+mx_idx) = exo_f_state;
    for j=l+mx_idx-1:-1:l+1,
      if signal(1,j) > ige_exo_threshold,
        state_seq(j) = exo_f_state;
      else
        state_seq(l+1:j) = STATES.ige;
        break
      end
    end
    for j=l+mx_idx+1:1:r-1,
      if signal(1,j) > ino_exo_threshold,
        state_seq(j) = exo_f_state;
      else
        state_seq(j:r-1) = ino_f_state;
        break
      end
    end
  else
    label_seq(l)
    label_seq(r)
    l
    r
    keyboard
  end
end
% check that only ambigous positions have been changed and unambigous
% ones were not affected by the above loop
unamb_idx = find(~isnan(unamb_state_seq));
assert(all(unamb_state_seq(unamb_idx)==state_seq(unamb_idx)));

% fill in first/second exon/intron states
exo_blocks = find_blocks(state_seq==exo_f_state);
for i=1:size(exo_blocks,2),
  state_seq(exo_blocks(1,i)+1:exo_blocks(2,i)) = exo_s_state;
end
ino_blocks = find_blocks(state_seq==ino_f_state);
for i=1:size(ino_blocks,2),
  state_seq(ino_blocks(1,i)+1:ino_blocks(2,i)) = ino_s_state;
end

% assert that all intron states are properly contained between exons
if ~isempty(ino_blocks),
  assert(all(find(state_seq==exo_f_state, 1, 'first') < ino_blocks(1,:)));
  assert(all(find(state_seq==exo_f_state, 1, 'last')  > ino_blocks(2,:)));  
end

assert(~any(isnan(state_seq)));
assert(state_model(state_seq(1)).is_start==1);
assert(state_model(state_seq(end)).is_stop==1);

% eof
