function state_model = specify_model(PAR)

% state_model = specify_model(PAR)
%
% User definition of the state-transition model.
%
% PAR -- a struct of parameters specified in setup_training.m and
%   train_hmsvm.m
% returns a preliminary version of the graphical model to be completed by
%   complete_model.m
%
% written by Georg Zeller, MPI Tuebingen, Germany, 2007-2008

%%% define state names and corresponding state ids
STATES = get_state_set_mSTAD(PAR);

% initialize names and ids in state_model struct
fn = fieldnames(STATES);
for i=1:length(fn),
  state_model(i).is_start = 0;
  state_model(i).is_stop  = 0;
end
state_model(STATES.ige).is_start = 1;
state_model(STATES.ige).is_stop  = 1;


%%% associate a label with each state
LABELS = get_label_set_mSTAD();
exo_f_idx = strmatch('exo_f_', fn);
assert(length(exo_f_idx) == PAR.num_levels);
exo_s_idx = strmatch('exo_s_', fn);
assert(length(exo_s_idx) == PAR.num_levels);

ino_f_idx = strmatch('ino_f_', fn);
assert(length(ino_f_idx) == PAR.num_levels);
ino_s_idx = strmatch('ino_s_', fn);
assert(length(ino_s_idx) == PAR.num_levels);

state_model(STATES.ige).label     = LABELS.intergenic;
for i=1:PAR.num_levels, 
  state_model(exo_f_idx(i)).label = LABELS.exonic;
  state_model(exo_s_idx(i)).label = LABELS.exonic;
  state_model(ino_f_idx(i)).label = LABELS.intronic;
  state_model(ino_s_idx(i)).label = LABELS.intronic;
end


%%% define allowed transitions
% successors contains all ids of states reachable via an arc from this state
% trans_scores indicates whether a transition score will be learned (if 1)
% or whether transition score will be fixed to 0 (if 0).
% 1s are later replaced by an index into transition_scores.
state_model(STATES.ige).successors       = [STATES.ige exo_f_idx'];
state_model(STATES.ige).trans_scores     = [1 ones(1,PAR.num_levels)];
for i=1:PAR.num_levels,
  state_model(exo_f_idx(i)).successors   = [STATES.ige exo_s_idx(i) ino_f_idx(i)]; 
  state_model(exo_f_idx(i)).trans_scores = [1 1 1];
  state_model(exo_s_idx(i)).successors   = [STATES.ige exo_s_idx(i) ino_f_idx(i)]; 
  state_model(exo_s_idx(i)).trans_scores = [1 1 1];

  state_model(ino_f_idx(i)).successors   = [ino_s_idx(i) exo_f_idx(i)];
  state_model(ino_f_idx(i)).trans_scores = [1 1];
  state_model(ino_s_idx(i)).successors   = [ino_s_idx(i) exo_f_idx(i)];
  state_model(ino_s_idx(i)).trans_scores = [1 1];
end
% transitions from level i to i+1 and vice versa
for i=1:PAR.num_levels-1,
  state_model(ino_f_idx(i)).successors(end+1)     = ino_s_idx(i+1);
  state_model(ino_f_idx(i)).trans_scores(end+1)   = 1;
  state_model(ino_f_idx(i+1)).successors(end+1)   = ino_s_idx(i);
  state_model(ino_f_idx(i+1)).trans_scores(end+1) = 1;
end


%%% specify whether feature scoring functions are learned
%%% expected is a 0/1 vector with nonzero entries for the features to be
%%% scored by functions included in the learning process
assert(PAR.num_features == 1);
state_model(STATES.ige).learn_scores     = [1]; % hyb intensity
for i=1:PAR.num_levels,
  state_model(exo_f_idx(i)).learn_scores = [1]; % hyb intensity
  state_model(exo_s_idx(i)).learn_scores = [1]; % hyb intensity
  state_model(ino_f_idx(i)).learn_scores = [1]; % hyb intensity
  state_model(ino_s_idx(i)).learn_scores = [1]; % hyb intensity
end


%%% specify whether scoring functions should be shared between several
%%% states as a matrix k x 2, where k is equal to the number of nonzeros
%%% in learn_scores of the same state
%%% first column is a feature index and second column indicates the state
%%% id  to which the scoring parameters correspond
for i=1:length(state_model),
  state_model(i).feature_scores ...
      = [find(state_model(i).learn_scores), i];
end


%%% specify monotonicity constraints for feature scoring functions
%%% as a vector of length k containing +1 (monotonically increasing
%%% scoring function), -1 (monotonically decreasing) and 0 (no
%%% monotonicity desired) entries where k is equal to the
%%% number of nonzeros in learn_scores of the same state.
%%% will not be considered when scoring functions are shared with
%%% another state
state_model(STATES.ige).monot_scores     = 0;
for i=1:PAR.num_levels,
  state_model(exo_f_idx(i)).monot_scores = 0;
  state_model(exo_s_idx(i)).monot_scores = 0;
  state_model(ino_f_idx(i)).monot_scores = 0;
  state_model(ino_s_idx(i)).monot_scores = 0;
end


%%% specify whether feature scoring functions will be coupled via
%%% regularization terms to those of other states as a k x 2 matrix where
%%% k is equal to the number of nonzeros in learn_scores of the same state. 
%%% first column is a feature index and second column indicates the state
%%% id to which the scoring parameters correspond (both should be zero
%%% if no coupling is desired)
%%% AVOID TO COUPLE the same pair of states twice as (i,j) and (j,i).
%%% only feature scoring functions which are not shared between states
%%% can be coupled
for i=1:length(state_model),
  state_model(i).score_coupling ...
      = zeros(sum(state_model(i).learn_scores),2);
end

% couple initial exon to second exon states (likewise for intron states)
for i=1:PAR.num_levels,
  state_model(exo_f_idx(i)).score_coupling = [1 exo_s_idx(i)]; 
  state_model(ino_f_idx(i)).score_coupling = [1 ino_s_idx(i)];
end

% couple exon and intron PLiFs across expression levels
for i=1:PAR.num_levels-1,
  state_model(exo_f_idx(i)).score_coupling = [1 exo_f_idx(i+1)]; 
  state_model(exo_s_idx(i)).score_coupling = [1 exo_s_idx(i+1)]; 

  state_model(ino_f_idx(i)).score_coupling = [1 ino_f_idx(i+1)];
  state_model(ino_s_idx(i)).score_coupling = [1 ino_s_idx(i+1)];
end

% eof