function train_exms = filter_train_exms(exm_id, label, train_exms, train_genes)

% train_exms = filter_train_exms(exm_id, label, train_exms, train_genes)
%
% Filters example sequences returning the example ids of the subset of
% sequences compatible with the state_model and other assumptions made
% for mSTAD training (e.g. first and last probe have to be labelled
% intergenic, no overlapping genes, terminal exons are covered by at
% least one probe etc., see comments below).
%
% exm_id -- a vector of N example IDs (pos. integers) indicating to
%   which example a label belongs to (e.g. 11111222 means that the first
%   example is a sequence of length 5 and is followed by the second
%   example sequence of length 3)
% label -- the probe label (vector of length N,
%   see get_label_set_mSTAD.m)
% train_exms -- the (unique) set of example IDs which will be used for
%   training
% train_genes -- a mask which is 0 for genes not useful for training
%   (e.g. because of overlap to other genes) and 1 elsewhere
% returns the example ids of the subset of sequences compatible with the
%   assumpations made for mSTAD training 
%
% see setup_training.m prepare_data.m
%
% written by Georg Zeller, MPI Tuebingen, Germany, 2007-2008

LABELS = get_label_set_mSTAD();

fprintf('Subselecting training examples...\n');
num_orig_exms = length(unique(train_exms));

idx = find(ismember(exm_id, train_exms));
exm_id = exm_id(idx);
label = label(idx);

% remove problematic examples from training set
rm_idx = zeros(size(exm_id));
for i=1:length(train_exms),
  idx = find(exm_id==train_exms(i));
  assert(~isempty(idx));
  
  % remove bad gene blocks, e.g. ones with overlapping genes
  if any(train_genes(idx) == 0),
    rm_idx(idx) = 1;
  end
  
  % first last probe has to be annotated as intergenic
  if label(idx(1)) ~= LABELS.intergenic,
    rm_idx(idx) = 1;
  end

  % last probe has to be annotated as intergenic
  if label(idx(end)) ~= LABELS.intergenic,
    rm_idx(idx) = 1;
  end
  
  % at least one probe has to be annotated as exonic
  if ~any(label(idx) == LABELS.exonic),
    rm_idx(idx) = 1;
  end
  
  % there have to be at least 10 informatively labeled probes
  if sum(label(idx) == LABELS.exonic ...
         | label(idx) == LABELS.intronic ...
         | label(idx) == LABELS.intergenic) < 10,
    rm_idx(idx) = 1;
  end
  
  % intron probes have to be properly contained between exon probes
  ino_blocks = find_blocks(label(idx)==LABELS.intronic);
  if ~isempty(ino_blocks),
    % intergenic and intronic probes next to each other
    if any(label(idx(ino_blocks(1,:))-1)==LABELS.intergenic),
      rm_idx(idx) = 1;
    end
    % intronic and intergenic probes next to each other
    if any(label(idx(ino_blocks(2,:))+1)==LABELS.intergenic),
      rm_idx(idx) = 1;
    end
  end
  
  if mod(1,1000)==1,
    fprintf('  %2.1f%%  \r', 100*i/length(train_exms));
  end
end
fprintf('  %2.1f%%\n', 100*i/length(train_exms));

exm_id(rm_idx==1) = [];
train_exms = unique(exm_id);

fprintf('  retained %i examples (%2.1f%%)...\n', length(train_exms), ...
        100*length(train_exms)/num_orig_exms);

% eof