% Starts HM-SVM training (calling train_hmsvm.m) for different
% combination of hyperparameters. Can thus be used for cross-validation
% and model selection.
%
% see HM-SVM toolbox for HM-SVM specifics.
%
% written by Georg Zeller, MPI Tuebingen, Germany, 2007-2008

% adjust directories in extend_path.m to make sure that all includes work
addpath ../
[mSTAD_home common_dir hmsvm_dir] = set_mSTAD_paths();

optimizer = 'mosek';
% path to the optimizer interface
opt_dir = sprintf('%s/opt_interface/%s', hmsvm_dir, optimizer);

% option to use rproc tools to submit training jobs to a cluster
USE_RPROC = 0;

% number of subsets used for cross-validation
num_xval_subsets = 5;

% names of parameters to be independently specified (possibly differently
% between training runs) 
param_names = {'C_small', ...
               'C_smooth', ...
               'C_coupling', ...
               'num_train_exm', ...
               'reg_type'...
               'train_subsets', ...
              };

% parameter combinations to be used for independent training
parameters = { ...
     [0.2],  [10], [2], [100], 'QP', [1]; ...
     [0.2],  [10], [2], [100], 'QP', [3]; ...
             };

% basic data directory
base_dir = '../../../PSB_data/';
% file name from which training data will be loaded
data_dir = [base_dir 'mSTAD/input_data/'];
data_file = [data_dir  'T_003_ABC_R_A_QN_RR_segm'];

% name of the file to which the trained HM-SVM will be written
out_dir = [base_dir 'mSTAD/training/mSTAD_result_' ...
           datestr(now,'yyyy-mm-dd_HHhMM')];
% name of the learning model
model_name = 'mSTAD_model';
% directory of the model-specific code
model_dir = sprintf('%s/%s', mSTAD_home, model_name);
if exist(model_name) ~= 7,
  addpath(model_dir);
end

% constant parameters
CONST_PAR.model_name = model_name;                 % name of the learning model
CONST_PAR.model_dir = model_dir;                   % model directory
CONST_PAR.data_file = data_file;                   % name of the training data file
CONST_PAR.num_levels = 10;                         % number of discrete expression levels
CONST_PAR.num_plif_nodes = CONST_PAR.num_levels+2; % number of supporting points
                                                   % for each scoring function
CONST_PAR.constraint_margin = 10;                  % use heuristic training procedure
CONST_PAR.optimizer = optimizer;                   % use CPLEX solver for training
% matlab functions from the following paths are needed during mSTAD training
CONST_PAR.include_paths = { ...
    model_dir, ...
    opt_dir, ...
    common_dir, ...
    }

% seed for random number generation
rand('seed', 11081979);

% partition exon intensities into expression bins 
% (i.e. assign discrete expression levels) 
load(data_file, 'signal', 'label', 'exm_id', 'train_genes', 'chunk_id', ...
     'NUM_EXPR_LEVELS');
assert(NUM_EXPR_LEVELS == CONST_PAR.num_levels);
CONST_PAR.expression_bins = discretize_expression(signal(1,:), label, ...
                                                  exm_id, CONST_PAR.num_levels);
assert(size(CONST_PAR.expression_bins,1) == CONST_PAR.num_levels);

% partition data for cross-validation
[chunk_subsets exm_subsets] = partition_data(chunk_id, exm_id, num_xval_subsets);

% start training independently for each parameter combination
JOB_INFO = [];
for i=1:size(parameters,1),
  PAR = CONST_PAR;
  PAR.out_dir = [out_dir '_model' num2str(i) '/']; % output directory
  
  % parameters for which the best model is selected
  fprintf('Training model %i...\n', i);
  for j=1:length(param_names),
    if ischar(parameters{i,j})
      fprintf('  %s = %s\n', param_names{j}, parameters{i,j});
    elseif length(parameters{i,j}) == 1,
      fprintf('  %s = %f\n', param_names{j}, parameters{i,j});
    else
      p_str = [];
      for k=1:length(parameters{i,j}),
        p_str = [p_str sprintf('%f ', parameters{i,j}(k))];
      end
      fprintf('  %s = %s\n', param_names{j}, p_str);
    end
    PAR = setfield(PAR, param_names{j}, parameters{i,j});
  end
  fprintf('\n\n');
  
  % assign example sequences to training, validation and test set
  assert(isfield(PAR, 'train_subsets'));
  PAR.vald_subsets = max(PAR.train_subsets)+1;
  if PAR.vald_subsets > num_xval_subsets,
    PAR.vald_subsets = mod(PAR.vald_subsets, num_xval_subsets);
  end
  assert(all(ismember(PAR.train_subsets, [1:num_xval_subsets])));
  assert(all(ismember(PAR.vald_subsets,  [1:num_xval_subsets])));
  PAR.test_subsets = setdiff(1:num_xval_subsets, ...
                             [PAR.train_subsets PAR.vald_subsets]);
  train_exms = [exm_subsets{PAR.train_subsets}];
  PAR.train_chunks = [chunk_subsets{PAR.train_subsets}];
  PAR.train_exms = filter_train_exms(exm_id, label, ...
                                     train_exms, train_genes);
  assert(all(ismember(PAR.train_exms, train_exms)));
  PAR.vald_chunks = [chunk_subsets{PAR.vald_subsets}];
  PAR.vald_exms  = [exm_subsets{PAR.vald_subsets}];
  
  PAR.test_chunks = [chunk_subsets{PAR.test_subsets}];
  PAR.test_exms  = [exm_subsets{PAR.test_subsets}];
  
  assert(isempty(intersect(PAR.train_exms, PAR.vald_exms)));
  assert(isempty(intersect(PAR.test_exms, [PAR.train_exms, PAR.vald_exms])));
  assert(isempty(intersect(exm_id(ismember(chunk_id, PAR.train_chunks)), ...
                           [PAR.vald_exms PAR.test_exms])));
  assert(length(PAR.train_exms) >= PAR.num_train_exm);
  
  disp(PAR)
  
  if USE_RPROC,
    % submit jobs to cluster and do training in parallel
    % RPROC settings
    RPROC_MEMREQ             = 2000;
    RPROC_OPT.express        = 0;
    RPROC_OPT.immediately_bg = 0;
    RPROC_OPT.immediately    = 0;
    RPROC_OPT.arch           = 64; % take only 64 bit nodes
    RPROC_OPT.identifier     = sprintf('mSTAD_m%i_',i);
    RPROC_OPT.start_dir      = hmsvm_dir;
    RPROC_TIME               = 12*(PAR.num_train_exm/100)*60; % mins

    JOB_INFO{end+1} = rproc('train_hmsvm', ...
                            PAR, RPROC_MEMREQ, RPROC_OPT, RPROC_TIME);
    fprintf('\nSubmitted job %i\n\n', length(JOB_INFO));
  else
    % train sequentially for different parameter combinations
    train_hmsvm(PAR);
  end
end

% eof