function [PM_I_n T1 T2] = init_prediction(PM_I, PM_I_transcript, probe_seq, probe_props, dist, OPT, method_str)

% [PM_I_n T1 T2] = init_prediction(PM_I, PM_I_transcript,
%                      probe_seq, probe_props, dist, OPT, method_str)
%
% Initializes data and calls prediction routines.
%
% PM_I -- vector of hybridization intensities to be normalized
% PM_I_transcript -- vector of idealized transcript intensities
% probe_seq -- a matrix of probe sequences corresponding to PM_I
% probe_props -- additional sequence property features computed from
%   probe sequences
% dist -- a field with transcript distances that is needed when also
%   normalizing for 3' transcript bias in intensity
% OPT -- a struct with parameters controlling the workflow
% methodstr -- a string specifying the normaliziation method, one of
%   'Ridge' or 'SVR'
% returns normalized intensities as well as evaluation structures
%   containing data on T1 and T2 evaluation metrics
%
% written by Georg Zeller, MPI Tuebingen, Germany, 2007-2008

if OPT.VERBOSE, fprintf('Loading predictors...\n'); end
  
PM_I_n = nan(size(PM_I));
predictor_idx = zeros(size(PM_I));

if ~isfield(OPT, 'SUBSETS_TO_USE') || isempty(OPT.SUBSETS_TO_USE),
  subsets = 1:OPT.NUM_SUBSETS;
else
  subsets = OPT.SUBSETS_TO_USE;
end
for i=subsets,
  rr_dr  = [OPT.RR_DIR num2str(i) '/'];
  svr_dr = [OPT.SVR_DIR num2str(i) '/'];
  clear P
  
  for p=1:OPT.NUM_PREDICTORS,
    switch method_str,
      case 'Ridge',
       fn_rr = sprintf('%strained_rr_fold%i_quant%i_lambda%1.5f_o%i.mat', ...
                       rr_dr, i, p, OPT.RR.lambda, OPT.RR.order);
       tmp = load(fn_rr, 'RR');
       tmp.PREDICTOR = tmp.RR;
       tmp = rmfield(tmp, 'RR');
     
       if ~equal(tmp.PREDICTOR.PARAM.fn_out, fn_rr),
         fprintf('Setting fn_out to %s\n', fn_rr);
         tmp.PREDICTOR.PARAM.fn_out = fn_rr;
       end
     case 'SVR',
       fn_svr = sprintf('%strained_svm_fold%i_quant%i_C%1.3f_tube%1.3f_o%i.mat', ...
                       svr_dr, i, p, OPT.SVR.C, OPT.SVR.tube, OPT.SVR.wdk_order);
       tmp = load(fn_svr, 'SVM');
       tmp.PREDICTOR = tmp.SVM;
       tmp = rmfield(tmp, 'SVM');
     
       if ~equal(tmp.PREDICTOR.PARAM.fn_out, fn_svr),
         fprintf('Setting fn_out to %s\n', fn_svr);
         tmp.PREDICTOR.PARAM.fn_out = fn_svr;
       end
    end
    if OPT.VERBOSE, fprintf('  Loaded predictor for %ith quantile...\n', p); end
    assert(equal(tmp.PREDICTOR.PARAM.annotation_tag, OPT.ANNOTATION_TAG));
    if isfield(OPT, 'DIST_CORRECTION'),
      assert(OPT.DIST_CORRECTION == tmp.PREDICTOR.DIST_CORRECTION);
    end
    
    if isfield(tmp.PREDICTOR, 'X_train_num'),
      tmp.PREDICTOR = rmfield(tmp.PREDICTOR, 'X_train_num');
    end
    
    if isequal(method_str, 'Ridge') ...
          & (~isfield(OPT, 'LOAD_RR_PRED') || ~OPT.LOAD_RR_PRED) ...
       | isequal(method_str, 'SVR') ...
          & (~isfield(OPT, 'LOAD_SVR_PRED') || ~OPT.LOAD_SVR_PRED),
      if isfield(tmp.PREDICTOR, 'X_eval_num'),
        tmp.PREDICTOR = rmfield(tmp.PREDICTOR, 'X_eval_num');
      end
      if isfield(tmp.PREDICTOR, 'Y_pred_eval'),
        tmp.PREDICTOR = rmfield(tmp.PREDICTOR, 'Y_pred_eval');
      end
      if isfield(tmp.PREDICTOR, 'Y_eval'),
        tmp.PREDICTOR = rmfield(tmp.PREDICTOR, 'Y_eval');
      end
      if isfield(tmp.PREDICTOR, 'Q1_VL'),
        tmp.PREDICTOR = rmfield(tmp.PREDICTOR, 'Q1_VL');
      end
      if isfield(tmp.PREDICTOR, 'Q2_VL'),
        tmp.PREDICTOR = rmfield(tmp.PREDICTOR, 'Q2_VL');
      end
    end
    P(p) = tmp.PREDICTOR;
    
    if ~isfield(OPT, 'USE_TEST_DATA') 
      if p==1,
        assert(all(predictor_idx(tmp.PREDICTOR.test_probe_idx{1}) == 0));
        predictor_idx(tmp.PREDICTOR.test_probe_idx{1}) = i;
      else
        assert(all(predictor_idx(tmp.PREDICTOR.test_probe_idx{1}) == i));
      end    
    elseif OPT.USE_TEST_DATA,
      if p==1,
        predictor_idx(tmp.PREDICTOR.test_probe_idx{1}) = i;
        % also take 2nd validation set for test purposes
        predictor_idx(tmp.PREDICTOR.vald_probe_idx{2}) = i;
      else
        assert(all(predictor_idx(tmp.PREDICTOR.test_probe_idx{1}) == i));
        assert(all(predictor_idx(tmp.PREDICTOR.vald_probe_idx{2}) == i));
      end
    else
      if p==1,
        assert(all(predictor_idx(tmp.PREDICTOR.vald_probe_idx{1}) == 0));
        predictor_idx(tmp.PREDICTOR.vald_probe_idx{1}) = i;
      else
        assert(all(predictor_idx(tmp.PREDICTOR.vald_probe_idx{1}) == i));
      end
    end
  end
  eval_idx = find(predictor_idx==i);
  
  if isfield(OPT, 'DIST_CORRECTION') && OPT.DIST_CORRECTION,
    for p=1:OPT.NUM_PREDICTORS,
      P(p).dist = dist(eval_idx);
      assert(all(P(p).dist>=0));
    end
  end
  
  fns = fieldnames(probe_props);
  seq_props = struct;
  for f=1:length(fns),
    tmp = getfield(probe_props, fns{f});                           
    seq_props = setfield(seq_props, fns{f}, tmp(:,eval_idx));
  end

  [I_n_fold, P] = predict_per_quantile(P, ...
                                       PM_I(eval_idx), ...
                                       probe_seq(:,eval_idx), ...
                                       seq_props, ...
                                       OPT, ...
                                       PM_I_transcript(eval_idx), ...
                                       method_str);

  if ~isfield(OPT, 'USE_TEST_DATA') || ~OPT.USE_TEST_DATA,
    assert(all(isnan(PM_I_n(eval_idx))));
  end
  assert(~any(isnan(I_n_fold) & ~isnan(PM_I(eval_idx))));
  PM_I_n(eval_idx) = I_n_fold;
  clear I_n_fold
  
  T1(i,:) = [P.T1];
  T2(i,:) = [P.T2];
  
  if OPT.VERBOSE,
    idx = find(~isnan(PM_I_n(eval_idx)) ...
               & PM_I_transcript(eval_idx) ~= -1);
    idx = eval_idx(idx);
    plot_idx = 1000:6000;
    figure
    hold on
    plot(PM_I(idx(plot_idx)), '.-k');
    plot(PM_I_transcript(idx(plot_idx)), '.-g');
    plot(PM_I_n(idx(plot_idx)), '.-r');     
    clear idx
  end
end

if ~isfield(OPT, 'SUBSETS_TO_USE') || isempty(OPT.SUBSETS_TO_USE),
  assert(all(predictor_idx~=0));
end
