% Applies sequence normalization methods to (whole-genome) test data and
% writes .cel-like flat files with normalized intensities
%
% written by Georg Zeller, MPI Tuebingen, Germany, 2007-2008

set_paths

% switches to apply sequence quantile normalization (SQN), support vector
% regression (SVR) or Ridge regression (RR)
OPT.APPLY_SQN        = 0;
OPT.APPLY_SVR        = 0;
OPT.APPLY_RR         = 1;

% length of oligonucleotide probes
OPT.PROBE_LEN       = 25;
% probe labelling according to annotation
OPT.LABEL           = get_label_set();

% C hyperparameter of the SVM
OPT.SVR.C           = 10;
% maximal substring length
OPT.SVR.wdk_order   = 3;
% tolerance interval that is loss-free in training
OPT.SVR.tube        = 0.1;

% regularization strength
OPT.RR.lambda       = 0.001;
% maximal substring length
OPT.RR.order        = 3;

% number of subsets in data split
OPT.NUM_SUBSETS     = 5;
% number of quantiles for each of which a separate predictor is used
OPT.NUM_PREDICTORS  = 20;

% specifies the version of the gene annotation
OPT.ANNOTATION_TAG  = 'TAIR7';
% switch to save predicted labels with predictors
OPT.SAVE_PREDICTORS = 0;

% directories where predictors are stored
OPT.common_out_dir = '../../PSB_data/norm_probe_results/';
OPT.SVR_DIR = sprintf('%ssvr_result_2008-03-04_15h57_fold', OPT.common_out_dir);
OPT.RR_DIR  = sprintf('%srr_result_2008-03-04_13h09_fold', OPT.common_out_dir);
OPT.SQN_DIR = sprintf('%ssqn_2008-03-04_13h13', OPT.common_out_dir);

% directory where array data is stored
OPT.data_dir  = '../../PSB_data/array_data/';
% directory where mappings between array coordinates, genome and genes are stored
OPT.map_dir   = '../../PSB_data/maps/';

% file names of arrays for whole-genome prediction
OPT.fn_arrays = { ...
    'T_003_tA_R_A_QN', ...
    'T_003_tB_R_A_QN', ...
    'T_003_tC_R_A_QN' ...
                };

OPT.VERBOSE = 0;

OPT

% file with mapping between array coordinates and genome location 
fn_genomic_probes = [OPT.map_dir 'genomic_probes'];
% file with mapping between array coordinates and genes
fn_genic_probes   = [OPT.map_dir 'genic_probes_' OPT.ANNOTATION_TAG];


%%%%%%%%%%%%%%%%%%%%%%%%
%%%   prepare data   %%%
%%%%%%%%%%%%%%%%%%%%%%%%

% get mapping probes -> genome
fprintf('Loading maping probes -> genome...\n');
load(fn_genomic_probes, 'probe_id', 'probe_seq', 'probe_props', ...
     'probe_PM_array_idx');  

% get mapping probes -> genes
fprintf('Loading mapping probes <-> genes...\n');
load(fn_genic_probes, 'probe_label', 'genes');  

% get intensities
fprintf('Loading array intensities...\n');
for f=1:length(OPT.fn_arrays),
  tmp = load([OPT.data_dir OPT.fn_arrays{f}]);
  arrays(f) = tmp.array;
end
clear tmp

probe_PM_I = zeros(length(OPT.fn_arrays),length(probe_id));

for p=1:length(probe_PM_array_idx),
  for f=1:length(OPT.fn_arrays),
    probe_PM_I(f,p) = arrays(f).mean(probe_PM_array_idx(p));
 end
  if OPT.VERBOSE && mod(p,10000)==0, fprintf('  %i\r',p); end
end
if OPT.VERBOSE, fprintf('\n'); end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%   log-transform intensities   %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% log_2 transform intensities
PM_I = log2(probe_PM_I);
clear probe_PM_I


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%   apply normalizations   %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for f=1:length(OPT.fn_arrays),
  [genes, PM_I_transcript] = median_transcript_intensities(PM_I(f,:), ...
                                    probe_id, probe_label, genes, OPT.LABEL);
  dist = zeros(size(PM_I_transcript));
  
  if OPT.APPLY_SQN,
    % load sequence-based quantile normalized intensities
    tmp = load([OPT.SQN_DIR '/I_n_iter100.mat'], 'I_n');
    PM_I_n_SQN = tmp.I_n;

    idx = find(~isnan(PM_I_n_SQN) & PM_I_transcript ~= -1);
    T1 = mean(abs(PM_I_n_SQN(idx) - PM_I_transcript(idx))) ...
         / mean(abs(PM_I(f,idx) - PM_I_transcript(idx)))
    T2 = mean((PM_I_n_SQN(idx) - PM_I_transcript(idx)).^2) ...
         / mean((PM_I(f,idx) - PM_I_transcript(idx)).^2)
  
    % write normalized data to files
    fn_array = [OPT.data_dir OPT.fn_arrays{f}];
    arrays = reconstruct_array(fn_array, 'SQN', 2.^PM_I_n_SQN, probe_id, ...
                               'sequence quantile-normalization');
  end

  if OPT.APPLY_SVR,
    % do SVR based normalization
    [PM_I_n_SVR T1 T2] = init_prediction(PM_I(f,:), PM_I_transcript, ...
                                         probe_seq, probe_props, dist, OPT, ...
                                         'SVR');

    idx = find(~isnan(PM_I_n_SVR) & PM_I_transcript ~= -1);
    T1 = mean(abs(PM_I_n_SVR(idx) - PM_I_transcript(idx))) ...
         / mean(abs(PM_I(f,idx) - PM_I_transcript(idx)))
    T2 = mean((PM_I_n_SVR(idx) - PM_I_transcript(idx)).^2) ...
         / mean((PM_I(f,idx) - PM_I_transcript(idx)).^2)

    % write normalized data to files
    fn_array = [OPT.data_dir OPT.fn_arrays{f}];
    arrays = reconstruct_array(fn_array, 'SVR', 2.^PM_I_n_SVR, probe_id, ...
                                   'support vector regression');
  end
 
  if OPT.APPLY_RR,
    % do Ridge regression-based normalization
    [PM_I_n_RR T1 T2] = init_prediction(PM_I(f,:), PM_I_transcript, ...
                                        probe_seq, probe_props, dist, OPT, ...
                                        'Ridge');
    
    idx = find(~isnan(PM_I_n_RR) & PM_I_transcript ~= -1);
    T1 = mean(abs(PM_I_n_RR(idx) - PM_I_transcript(idx))) ...
         / mean(abs(PM_I(f,idx) - PM_I_transcript(idx)))
    T2 = mean((PM_I_n_RR(idx) - PM_I_transcript(idx)).^2) ...
         / mean((PM_I(f,idx) - PM_I_transcript(idx)).^2)
  
    % write normalized data to files
    fn_array = [OPT.data_dir OPT.fn_arrays{f}];
    arrays = reconstruct_array(fn_array, 'RR', 2.^PM_I_n_RR, probe_id, ...
                                   'Ridge regression');
  end
end



