% Starts training (and model selection) for transcript normalization
%
% written by Georg Zeller, MPI Tuebingen, Germany

set_paths

% switches to train sequence quantile normalization (SQN), support vector
% regression (SVR) or Ridge regression (RR)
OPT.APPLY_SQN       = 0;
OPT.TRAIN_SVR       = 0;
OPT.TRAIN_RR        = 1;

% switch to use rproc to submit cluster jobs
OPT.USE_RPROC       = 0;

% length of oligonucleotide probes
OPT.PROBE_LEN       = 25;
% probe labelling according to annotation
OPT.LABEL           = get_label_set();

%%% Support vector regression parameters
% parameters evaluated in model selection
  % C hyperparameter of the SVM
OPT.SVR.MOD_SEL_PARAM.C     = [10]; %[1 10 100];
  % tolerance interval that is loss-free in training
OPT.SVR.MOD_SEL_PARAM.tube  = [0.1]; %[0.2 0.1 0.05];
  % maximal substring length
OPT.SVR.MOD_SEL_PARAM.order = [3];

% constant parameters
  % numerical training accuracy
OPT.SVR.PARAM.svm_eps       = 1e-3;
  % number of mismatches considered by the weighted degree kernel
OPT.SVR.PARAM.wdk_mismatch  = 0;
  % maximal positional shift considered by the weighted degree kernel
OPT.SVR.PARAM.wdk_shift_vec = ones(1, OPT.PROBE_LEN);
OPT.SVR.PARAM.cache_size    = 400;

%%% Ridge regression parameters
% parameters evaluated in model selection
  % regularization strength
OPT.RR.MOD_SEL_PARAM.lambda = [10^-3];

% constant parameters
% maximal substring length
OPT.RR.PARAM.order          = 3;

% number of subsets for crossvalidation
OPT.NUM_SUBSETS     = 5;
% option to restrict the subsets for crossvalidation
OPT.SUBSETS_TO_USE  = [];
% number of quantiles each of which is used by a separate predictor
OPT.NUM_PREDICTORS  = 20;

% option to also consider transcript distance features to correct for
% intensity bias caused by oligo-dT amplificaton
OPT.DIST_CORRECTION = 0;
% specifies the version of the gene annotation
OPT.ANNOTATION_TAG  = 'TAIR7';

% directory where array data is stored
OPT.data_dir  = '../../PSB_data/array_data/';
% directory where mappings between array coordinates, genome and genes are stored
OPT.map_dir   = '../../PSB_data/maps/';
% file names of arrays to train on 
% (if more than 1 is given, intensities are averaged)
OPT.fn_arrays = { ... 
%    'T_003_tA_R_A_QN', ...
    'T_003_tB_R_A_QN', ...
%    'T_003_tC_R_A_QN' ...
                };

% timestamp appended to output directories
dt = datestr(now,'yyyy-mm-dd_HHhMM');
%dt = '2007-11-25_11h00';

% directories where output is written to
OPT.common_out_dir = '../../PSB_data/norm_probe_results/';
OPT.out_dir_SQN = sprintf('%ssqn_result_%s/', OPT.common_out_dir, dt);
if ~exist(OPT.out_dir_SQN, 'dir') & OPT.APPLY_SQN,
  mkdir(OPT.out_dir_SQN);
end
OPT.out_dir_SVR = sprintf('%ssvr_result_%s_fold', OPT.common_out_dir, dt);
OPT.out_dir_RR = sprintf('%srr_result_%s_fold',   OPT.common_out_dir, dt);

% option to load annotated array data to save time
OPT.LOAD_ARRAY_DATA = 0;

OPT.VERBOSE = 1;

OPT

% file with mapping between array coordinates and genome location 
fn_genomic_probes = [OPT.map_dir 'genomic_probes'];
% file with mapping between array coordinates and genes
fn_genic_probes   = [OPT.map_dir 'genic_probes_' OPT.ANNOTATION_TAG];


%%%%%%%%%%%%%%%%%%%%%%%%
%%%   prepare data   %%%
%%%%%%%%%%%%%%%%%%%%%%%%

if ~OPT.LOAD_ARRAY_DATA,
  % get mapping probes -> genome
  fprintf('Loading maping probes -> genome...\n\n');
  load(fn_genomic_probes, 'probe_id', 'probe_seq', 'probe_props', ...
       'probe_PM_array_idx');  

  % get mapping probes -> genes
  fprintf('Loading mapping probes <-> genes...\n\n');
  load(fn_genic_probes, 'probe_label', 'genes');  
  
  % get intensities
  fprintf('Loading array intensities...\n\n');
  for f=1:length(OPT.fn_arrays),
    tmp = load([OPT.data_dir OPT.fn_arrays{f}]);
    arrays(f) = tmp.array;
  end
  clear tmp

  probe_PM_I = zeros(length(OPT.fn_arrays),length(probe_id));
  
  for p=1:length(probe_PM_array_idx),
    for f=1:length(OPT.fn_arrays),
      probe_PM_I(f,p) = arrays(f).mean(probe_PM_array_idx(p));
    end
    if mod(p,10000)==0, fprintf('  %i\r',p); end
  end
  fprintf('\n');

  LABEL = OPT.LABEL;
  PM_tmp = probe_PM_I;
  for f=1:length(OPT.fn_arrays),
    probe_PM_I = PM_tmp(f,:);
    save([OPT.data_dir OPT.fn_arrays{f} '_'  OPT.ANNOTATION_TAG], ...
         'probe_id', 'probe_PM_I', 'probe_seq', 'probe_props', 'probe_label', ...
         'genes', 'LABEL');
  end
  probe_PM_I = PM_tmp;
  clear PM_tmp f i arrays probe_PM_array_idx
else
  for f=1:length(OPT.fn_arrays),
    if f==1,
      fprintf('loading data for array %s...\n', OPT.fn_arrays{f});
      tmp = load([OPT.data_dir OPT.fn_arrays{f} '_' OPT.ANNOTATION_TAG]);
      probe_id         = tmp.probe_id;
      probe_seq        = tmp.probe_seq;
      probe_PM_I       = tmp.probe_PM_I;
      probe_label      = tmp.probe_label;
      genes            = tmp.genes;
      probe_props      = tmp.probe_props;
    else
      fprintf('loading data for array %s...\n', OPT.fn_arrays{f});
      tmp = load([OPT.data_dir OPT.fn_arrays{f} '_' OPT.ANNOTATION_TAG], ...
                 'probe_PM_I');
      probe_PM_I = [probe_PM_I; tmp.probe_PM_I];
    end
  end
  clear tmp f
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%   log-transform and summarize intensities across replicate arrays   %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% log_2 transform intensities
PM_I = log2(probe_PM_I);
if length(OPT.fn_arrays)>1,
  fprintf('summarizing intensities across replicates...\n');
  [PM_I PM_num_replic] = summ_replicates(PM_I)
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%   "learn" various normalizations   %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% (i) quantile normalization for sequence bias

if OPT.APPLY_SQN,
  fprintf('Starting sequence-based quantile normalization...\n');
  
  SQN.I              = PM_I;
  SQN.probe_seq      = probe_seq;
  SQN.out_dir        = OPT.out_dir_SQN;
  SQN.num_iterations = 100;
  SQN.fn_arrays      = OPT.fn_arrays;
  
  % quantile-normalize all probes on the array
  seq_quantile_norm(SQN)
end

% (ii-iii) train a bunch of regressors conditional on output quantile
% for regression either SVR or Ridge regression is used
if OPT.TRAIN_SVR | OPT.TRAIN_RR,
  if OPT.DIST_CORRECTION,
    [genes, PM_I_transcript, dist] ...
        = transcript_intensities_3p_bias(PM_I, probe_id, probe_label, ...
                                         genes, OPT.LABEL);
  else
    [genes, PM_I_transcript] ...
        = median_transcript_intensities(PM_I, probe_id, probe_label, ...
                                      genes, OPT.LABEL);
  end

  
  fprintf('Starting regression...\n');

  % compute data partition for crossvalidation
  subset_idx = partition_index(probe_label, OPT.NUM_SUBSETS, OPT.LABEL);
  
  if isempty(OPT.SUBSETS_TO_USE),
    subsets = 1:OPT.NUM_SUBSETS;
  else
    subsets = OPT.SUBSETS_TO_USE;
  end
  for i=subsets,
    train_subsets = [i mod(i, OPT.NUM_SUBSETS)+1];
    vald_subsets  = [mod(i+1, OPT.NUM_SUBSETS)+1 mod(i+2, OPT.NUM_SUBSETS)+1];
    test_subsets  = mod(i+3, OPT.NUM_SUBSETS)+1;

    train_probe_idx = find(ismember(subset_idx, train_subsets));
    for j=1:length(vald_subsets),
      vald_probe_idx{j} = find(subset_idx==vald_subsets(j));
    end
    for j=1:length(test_subsets),
      test_probe_idx{j} = find(subset_idx==test_subsets(j));
    end

    % for training use only unambiguous probes
    is_good = probe_label(train_probe_idx)~=OPT.LABEL.ambiguous ...
              & ~isnan(PM_I(train_probe_idx)) ...
              & PM_I_transcript(train_probe_idx) ~= -1;

    P.fold_id = i;
    P.train_subsets    = train_subsets;
    P.train_probe_idx  = train_probe_idx;
    P.train_select_idx = is_good;
    P.vald_subsets     = vald_subsets;
    P.vald_probe_idx   = vald_probe_idx;
    P.test_subsets     = test_subsets;
    P.test_probe_idx   = test_probe_idx;
    
    P.DIST_CORRECTION = OPT.DIST_CORRECTION;
    if OPT.DIST_CORRECTION,
      P.dist = dist(train_probe_idx(is_good));
      assert(all(P.dist>=0));
      % implicit PLiF for 3p transcript bias
      s_dist = 100;
      % supporting points (x-values)
      P.dist_PLiF_s_x = 0:s_dist:2000;
    end
    
    P.PARAM.annotation_tag = OPT.ANNOTATION_TAG;
    P.PARAM.fn_arrays = OPT.fn_arrays;
    
    fns = fieldnames(probe_props);
    seq_props = struct;
    for f=1:length(fns),
      tmp = getfield(probe_props, fns{f});                           
      seq_props = setfield(seq_props, fns{f}, tmp(:, train_probe_idx(is_good)));
    end
    
    if OPT.TRAIN_SVR,
      P.PARAM.out_dir = [OPT.out_dir_SVR num2str(P.fold_id) '/'];
      if ~exist(P.PARAM.out_dir, 'dir') & OPT.TRAIN_SVR,
        mkdir(P.PARAM.out_dir);
      end
      disp(P);
      SVMs = train_per_quantile(P, ...
                                PM_I(train_probe_idx(is_good)), ...
                                PM_I_transcript(train_probe_idx(is_good)), ...
                                probe_seq(:,train_probe_idx(is_good)), ...
                                seq_props, ...
                                OPT, 'SVR');
    end    
    if OPT.TRAIN_RR,
      fprintf('Starting Ridge regression...\n');
      P.PARAM.out_dir = [OPT.out_dir_RR num2str(P.fold_id) '/'];
      if ~exist(P.PARAM.out_dir, 'dir') & OPT.TRAIN_RR,
        mkdir(P.PARAM.out_dir);
      end
      disp(P);
      RRs = train_per_quantile(P, ...
                               PM_I(train_probe_idx(is_good)), ...
                               PM_I_transcript(train_probe_idx(is_good)), ...
                               probe_seq(:,train_probe_idx(is_good)), ...
                               seq_props, ...
                               OPT, 'Ridge');
    end
    clear fns tmp seq_props
  end
end