function subset_idx = partition_index(probe_label, num_subsets, LABEL)

% subset_idx = partition_index(probe_label, num_subsets, LABEL)
%
% Partitions the set of probes into chunks of ~10k probes and assigns a
% random subset from {1,...,num_subsets} to each chunk. Splitting in
% genic regions is avoided.
%
% subset_idx = partition_index(probe_label, num_subsets, LABEL)
% probe_label -- a global annotation label for each probe (should be
%   equal to LABEL.ambiguous in case of e.g. overlapping genes)
% num_subsets -- the desired number of subsets
% LABEL -- a struct with all possible probe label constants
% returns an index of the same size as probe_label with values from 
%   {1,...,num_subsets}
%
% written by Georg Zeller, MPI Tuebingen, Germany, 2007-2008

rand('state', 08119179);
subset_idx = zeros(size(probe_label));

chunk_len   = 10000;

len         = length(probe_label);
num_chunks  = round(len / chunk_len);
offsets     = round((0:num_chunks-1) / num_chunks * len);
fprintf('Partitioning probes into %i chunks of length ~%i...\n',...
        num_chunks, chunk_len);  

% there should be at least 10 intergenic probes upstream 
% and downstream of a gene before splits
min_spacer = 10; 

genic_idx = find(probe_label~=LABEL.intergenic);

for b = 2:(num_chunks),
%  fprintf('chunks %3i|%3i, tentative boundary %i\n', (b-1), b, offsets(b));
  
  orig_offset =  offsets(b);
  
  % push split away from genic probes
  dists = offsets(b) - genic_idx;
  dists_up   = sort( dists(dists >= 0));
  dists_down = sort(-dists(dists <= 0));
  if min(dists_up) < min_spacer | min(dists_down) <= min_spacer,
    % how far to move upstream/downstream?
    t = 1;
    while dists_up(t+1)-dists_up(t) <= 2*min_spacer,
      t = t + 1;
    end
    new_dist_up = dists_up(t) + min_spacer;
    t = 1;
    while dists_down(t+1)-dists_down(t) <= 2*min_spacer,
      t = t + 1;
    end
    new_dist_down = dists_down(t) + min_spacer;
    % select direction of smallest distortion
    if new_dist_down < new_dist_up 
      offsets(b) = offsets(b) + new_dist_down;
    else
      offsets(b) = offsets(b) - new_dist_up - 1;
    end
  end
  dists = offsets(b) - genic_idx;
%  fprintf('                final boundary     %i\n', offsets(b));
%  probe_label(offsets(b)-2*min_spacer+1:offsets(b)-min_spacer)
%  probe_label(offsets(b)-min_spacer+1:offsets(b))
%  probe_label(offsets(b)+1:offsets(b)+min_spacer)
%  probe_label(offsets(b)+min_spacer+1:offsets(b)+2*min_spacer)
  assert(all(probe_label(offsets(b)-min_spacer+1:offsets(b))==LABEL.intergenic));
  assert(all(probe_label(offsets(b)+1:offsets(b)+min_spacer)==LABEL.intergenic));
end
%fprintf('last chunk [%i,%i], (length %i)\n', offsets(end)+1, len, len-offsets(end));

rand_subsets = [];
for i=1:num_subsets,
  rand_subsets = [rand_subsets i*ones(1,ceil(num_chunks/num_subsets))];
end
rand_subsets = rand_subsets(randperm(length(offsets)));
rand_subsets = rand_subsets(1:num_chunks);

for b=2:num_chunks,
  subset_idx(offsets(b-1)+1:offsets(b)) = rand_subsets(b-1);
end
subset_idx(offsets(end)+1:end) = rand_subsets(end);

for i=1:num_subsets,
  fprintf('Number of probes in subset %i: %7i\n', i, sum(subset_idx==i));
end

assert(~any(subset_idx==0));