function fh = plot_gc_dependance(I, seq, I_label, probe_label, LABEL);

% fh = plot_gc_dependance(I, seq, I_label, probe_label, LABEL);
%
% Plots median intensities binning probes according to their GC content.
%
% I -- an intensity vector
% seq -- a corresponding char matrix of probe sequences
% I_label -- a string label used in the figure legend
% probe_label -- annotation labels, one per probe (same size as I)
% LABEL -- a struct with all possible probe label constants
% returns a figure handle with the plot
%
% written by Georg Zeller, MPI Tuebingen, Germany, 2007-2008

if ~exist('I_label', 'var'), 
  I_label = 'intensity'; 
end
PROBE_LEN = size(seq,1); % oligo probe length

gc_content = sum(seq=='C' | seq=='G');
median_I = zeros(1,PROBE_LEN+1);
freq = zeros(1,PROBE_LEN+1);
if exist('probe_label', 'var') & exist('LABEL', 'var'),
  segment_freq = zeros(3,PROBE_LEN+1);
end

for i=1:PROBE_LEN+1,
  idx = find(gc_content==i-1);

  freq(i) = length(idx)/length(I);
  if exist('probe_label', 'var') & exist('LABEL', 'var'),
    segment_freq(1,i) = sum(probe_label(idx)==LABEL.exonic)/length(I);
    segment_freq(2,i) = sum(probe_label(idx)==LABEL.intronic)/length(I);
    segment_freq(3,i) = sum(probe_label(idx)==LABEL.intergenic)/length(I);
  end

  I_tmp = I(idx);
  I_tmp = I_tmp(~isnan(I_tmp));
  if ~isempty(I_tmp),
    median_I(i) = median(I_tmp);
  end
end

figure
hold on
bar(freq*100, 'FaceColor', [0.8 0.8 0.8]);
if exist('probe_label', 'var') & exist('LABEL', 'var'),
  bar(segment_freq'*100, 'stack');
  legend('ambiguous', 'exonic', 'intronic', 'intergenic', ...
         'Location', 'NorthEast');
end

style = '+';
switch I_label,
 case 'raw I',
  style = '+';
 case 'SVR-normalized I',
  style = 'p';
 case 'RR-normalized I',
  style = 'o';
 case 'SQN-normalized I',
  style = 'd';
end
idx = find(freq>10^-6);
plot(idx, median_I(idx), [style 'k'], 'MarkerSize', 10, 'LineWidth', 2);
xlabel('GC content');
ylabel(sprintf('Median log_2 %s / frequency (%%)', I_label));
title(sprintf('GC content-dependance of PM %s median', I_label));

print_dir = '../../PSB_data/norm_probe_results/figs/';
fh = gcf;
print(fh, '-depsc', '-r300', sprintf('%sGC_vs_PM_median_%s.eps', print_dir, ...
                                     I_label));

