function seq_quantile_norm(PAR)

% seq_quantile_norm(PAR)
%
% Following T.E. Royce et al, Assessing the need for sequence-based
% normalization in tiling microarray experiments, Bioinformatics (2007).
% Code adopted from % http://tiling.gersteinlab.org/sequence_effects/
%                                    sequence_normalization_functions.R
% Extension: Instead of a uniform ditribution with y-values in [0,1], the
% original intensities are normalized.
%
% PAR -- a struct with the following fields:
%   probe_seq, an array (25 x n) containing probe sequences
%   I, an array (1 x n) containing tiling array signals 
%      (log_2 transformed intensities)
%   out_dir
%   num_iterations
%
% written by Georg Zeller, MPI Tuebingen, Germany, 2007-2008

set_paths

VERBOSE = 0;

PROBE_LEN = size(PAR.probe_seq,1); % oligo probe length

nucl_idx = struct;
for i=1:PROBE_LEN,
  nucl_idx(i).A = find(PAR.probe_seq(i,:)=='A');
  nucl_idx(i).C = find(PAR.probe_seq(i,:)=='C');
  nucl_idx(i).G = find(PAR.probe_seq(i,:)=='G');
  nucl_idx(i).T = find(PAR.probe_seq(i,:)=='T');
end

if VERBOSE,
  fh1 = plot_gc_dependance(PAR.I, PAR.probe_seq);
  fh2 = plot_pos_quantile(PAR.I, nucl_idx, PROBE_LEN, 50);
  fh3 = plot_pos_quantile(PAR.I, nucl_idx, PROBE_LEN, 90);
  pause(1);
  figure
end

% number of supporting points for distribution approximation
n_support = 200;

% uniform position weights
pos_weights = ones(1,PROBE_LEN)/PROBE_LEN; 

I_n = PAR.I;
for i=1:PAR.num_iterations,
  I_mean = zeros(size(PAR.I));
  fprintf('normalized positions');
  for j=1:PROBE_LEN,
    [I_A perm_idx_A] = sort(I_n(nucl_idx(j).A));
    num_idx_A = find(~isnan(I_A));
    I_A = I_A(num_idx_A);
    [x_A, y_A] = approx_distr(I_A, n_support);
    
    [I_C perm_idx_C] = sort(I_n(nucl_idx(j).C));
    num_idx_C = find(~isnan(I_C));
    I_C = I_C(~isnan(I_C));
    [x_C, y_C] = approx_distr(I_C, n_support);
    
    [I_G perm_idx_G] = sort(I_n(nucl_idx(j).G));
    num_idx_G = find(~isnan(I_G));
    I_G = I_G(~isnan(I_G));
    [x_G, y_G] = approx_distr(I_G, n_support);
    
    [I_T perm_idx_T] = sort(I_n(nucl_idx(j).T));
    num_idx_T = find(~isnan(I_T));
    I_T = I_T(num_idx_T);
    [x_T, y_T] = approx_distr(I_T, n_support);

    assert(all(x_A == x_C & x_A == x_G & x_A == x_T));
    mean_x = x_A;
    mean_y = (y_A + y_C + y_G + y_T) / 4;

    I_tmp = reconstruct_distr(mean_x, mean_y, length(I_A));
    I_A = nan(size(nucl_idx(j).A));
    I_A(perm_idx_A(num_idx_A)) = I_tmp;
    
    I_tmp = reconstruct_distr(mean_x, mean_y, length(I_C));
    I_C = nan(size(nucl_idx(j).C));
    I_C(perm_idx_C(num_idx_C)) = I_tmp;
    
    I_tmp = reconstruct_distr(mean_x, mean_y, length(I_G));
    I_G = nan(size(nucl_idx(j).G));
    I_G(perm_idx_G(num_idx_G)) = I_tmp;

    I_tmp = reconstruct_distr(mean_x, mean_y, length(I_T));
    I_T = nan(size(nucl_idx(j).T));
    I_T(perm_idx_T(num_idx_T)) = I_tmp;
    
    I_mean(nucl_idx(j).A) = I_mean(nucl_idx(j).A) + I_A*pos_weights(j);
    I_mean(nucl_idx(j).C) = I_mean(nucl_idx(j).C) + I_C*pos_weights(j);
    I_mean(nucl_idx(j).G) = I_mean(nucl_idx(j).G) + I_G*pos_weights(j);
    I_mean(nucl_idx(j).T) = I_mean(nucl_idx(j).T) + I_T*pos_weights(j);

    fprintf(' %i', j);
  end
  fprintf('\nfinished iteration %i\n', i);
  d = (I_n(~isnan(I_n)) - I_mean(~isnan(I_mean)));
  squared_diff = dot(d, d)

  I_n = I_mean;
  if i==1 | mod(i,10)==0,
    if VERBOSE,
      fh1 = plot_gc_dependance(I_n, PAR.probe_seq);
      pause(1);
    end
    
    fname = sprintf('%sI_n_iter%i',PAR.out_dir,i);
    fprintf('Saving intermediate result to %s...\n',fname); 
    save(fname, 'PAR', 'I_n');
  end
end

  
  
