function [gene_blocks good_blocks] = make_gene_blocks(probe_id, probe_chr, gene_id, intergenic_blocks)

% [gene_blocks good_blocks] = make_gene_blocks(probe_id, probe_chr, gene_id, intergenic_blocks)
%
% Partitions chromosomal data into blocks corresponding to single genes
% by cutting in intergenic regions. A block will contain only a single
% gene unless genes overlap or have no intergenic probes between them.
%
% probe_id -- genomic locations (chr*10^8+position) of each probe (length N)
% probe_chr -- the chromosome of each probe's location (length N)
% gene_id -- a cell of length N containing all gene ids overlapping a probe
% intergenic_blocks -- intergenic regions given as a k x 2 matrix (first
%   column indicates region start, second one region end)
% returns a vector of N natural numbers (gene_blocks) indicating the gene
%   blocks (all probes belonging to the some block have the same block
%   id), further a 0/1 indicator (good_blocks) of whether blocks only
%   contain a single gene (the only ones useful for training)
%
% written by Georg Zeller, MPI Tuebingen, Germany, 2007-2008

gene_blocks = nan(size(probe_id));
good_blocks = nan(size(probe_id));

masked_gene_id = zeros(size(probe_id));
for i=1:length(masked_gene_id),
  if length(gene_id{i})==1,
    masked_gene_id(i) = gene_id{i};
  elseif length(gene_id{i})>1,
    masked_gene_id(i) = gene_id{i}(1);
    good_blocks(i) = 0;
  end
  if mod(i,10000)==0, 
    fprintf('  %2.2f%%\r', 100*i/length(masked_gene_id));
  end
end
fprintf('  %2.2f%%\n', 100*i/length(masked_gene_id));

cnt_masked_blocks = 0;
head_idx = intergenic_blocks(1,1);
for i=2:size(intergenic_blocks,2)-1,
  b = intergenic_blocks(1,i);
  e = intergenic_blocks(2,i);
  if probe_chr(b) ~= probe_chr(e),
    tail_idx = find(probe_chr(b:e)==probe_chr(b));
    tail_idx = tail_idx(end);
    tail_idx = tail_idx + b - 1;
  else
    tail_idx = floor((b+e)/2);
  end
  g = unique(masked_gene_id(head_idx:tail_idx));
  g(g==0) = [];
  assert(length(g)>=1);
  assert(all(isnan(gene_blocks(head_idx:tail_idx))));
  
  if length(g)==1 && ~any(good_blocks(head_idx:tail_idx)==0),
    % mark as good only genes separated by intergenic probes from
    % adjacent genes
    good_blocks(head_idx:tail_idx) = 1;
    gene_blocks(head_idx:tail_idx) = g;
  else
    assert(length(g)>=1);
    % more than one gene not separated by intergenic probes
    good_blocks(head_idx:tail_idx) = 0;
    gene_blocks(head_idx:tail_idx) = g(1);
    cnt_masked_blocks = cnt_masked_blocks + 1;
  end
  
  head_idx = tail_idx + 1;
end

tail_idx = length(probe_id);
g = unique(masked_gene_id(head_idx:tail_idx));
g(g==0) = [];
assert(length(g)==1);
assert(all(isnan(gene_blocks(head_idx:tail_idx))));
gene_blocks(head_idx:tail_idx) = g;
good_blocks(head_idx:tail_idx) = 1;

assert(~any(isnan(gene_blocks)));
assert(~any(isnan(good_blocks)));

fprintf('masked probes in %i bad gene_blocks (%2.1f%% of probes)\n', ...
        cnt_masked_blocks, 100*mean(good_blocks==0));

% eof