function predict_genome(dirs, array_id, iteration)

% predict_genome(dirs, array_id, [iteration])
%
% Predicts transcriptionally active regions for the whole genome for the
% given array data.
%
% dirs -- directories from which the trained mSTAD instance will be loaded
% array_id -- identifier of the array for which whole-genome predictions
%   are to be made
% iteration --number of training iterations for the mSTAD model (leave
%   empty to use the mSTAD instance after convergence)
%
% written by Georg Zeller, MPI Tuebingen, Germany, 2007-2008

% adjust set_mSTAD_paths.m to point to the correct directories
addpath ../
set_mSTAD_paths();
set_hmsvm_paths();

% controls the amount of output
VERBOSE = 1;

% write a summary of the prediction process to a file
WRITE_SUMMARY_FILE = 1;

% seed for random number generation
rand('seed', 11081979);

assert(exist('dirs', 'var')==1);
assert(exist('array_id', 'var')==1);
% use mSTAD instance for which training has converged by default
if ~exist('iteration', 'var'),
  iteration = [];
end

LABELS = get_label_set_mSTAD();

base_dir = '../../../PSB_data/mSTAD/';
for d=1:length(dirs)
  dirs{d} = [base_dir 'training/' dirs{d}];
end  

% load annotation of repetitive probes to annotate predictions
repeat_dir = '../../../PSB_data/repeat_data/';
load([repeat_dir 'kmer_repeats.mat'],  'kmers_exact', ...
     'kmers_inexact', 'kmers_short');

% load array data
input_dir = [base_dir 'input_data/'];
array_fn = [input_dir array_id '_segm.mat'];
load(array_fn, 'signal', 'label', 'chunk_id', 'probe_id');
intensity = signal(1,:);

% open summary file
dt = datestr(now,'yyyy-mm-dd');
if WRITE_SUMMARY_FILE,
  out_dir = [base_dir 'prediction/' array_id '_' dt];
  if ~exist(out_dir, 'dir'),
    mkdir(out_dir);
  end
  [out_fid msg] = fopen([out_dir '/prediction.txt'], 'w');
  disp(msg);
  assert(out_fid ~= -1);
else
  out_fid = 1;
end

% determine which data chunks are in the test sets of the trained mSTAD
% instances (in order to avoid making genome predictions on training or
% validation data)
num_chunks = max(unique(chunk_id));
assert(isequal(unique(chunk_id), 1:num_chunks));
data_file = [];
model_dir = [];
test_chunks = zeros(length(dirs),  num_chunks);
for p=1:length(dirs),
  if isempty(iteration)
    fn = sprintf('%slsl_final.mat', dirs{p});
  else
    fn = sprintf('%slsl_iter%i.mat', dirs{p}, iteration);
  end
  predictors(p) = load(fn, 'PAR', 'score_plifs', 'transition_scores', 'state_model');

  if VERBOSE>=1,
    fprintf(out_fid, 'Loaded predictor from %s...\n\n', fn);
    disp(predictors(p).PAR);
  end
  if p==1,
    data_file = predictors(p).PAR.data_file;
    model_dir = predictors(p).PAR.model_dir;
  end
  assert(isequal(data_file, predictors(p).PAR.data_file));
  assert(isequal(model_dir, predictors(p).PAR.model_dir));

  test_chunks(p,predictors(p).PAR.test_chunks) = 1;
end
% include path to mSTAD model
addpath(model_dir)

% assign predictor at random where test chunks overlap
idx = find(sum(test_chunks)>1);
for j=1:length(idx),
  p = find(test_chunks(:,idx(j)));
  p = p(randperm(length(p)));
  p = p(1);
  single_p = zeros(length(predictors),1);
  single_p(p) = 1;
  test_chunks(:,idx(j)) = single_p;
end
assert(all(sum(test_chunks)==1));

% do predictions on data chunks
q = 0;
fprintf(out_fid, 'Predicting on data from %s\n', array_fn);  
for p=1:length(predictors),
  if VERBOSE>=2,
    fprintf(out_fid, '  Using predictor %i for chunks ...\n', p);
  end
  chunks = find(test_chunks(p,:));
  transition_scores = predictors(p).transition_scores;
  score_plifs = predictors(p).score_plifs;
  PAR = predictors(p).PAR;
  % include HMSVM-specified paths
  if isfield(PAR, 'include_paths'),
    for i=1:length(PAR.include_paths),
      addpath(PAR.include_paths{i});
    end
  end
  
  for j=1:length(chunks),
    c = chunks(j);
    assert(~any(PAR.train_chunks==c) & ~any(PAR.vald_chunks==c));
    idx = find(chunk_id==c);
    if size(signal,1) == 1,
      idx_no_nan = find(~isnan(signal(:,idx)));
    else
      idx_no_nan = find(~any(isnan(signal(:,idx))));
    end
    trash = length(idx)-length(idx_no_nan);
    if VERBOSE>=1 && trash>0,
      fprintf(out_fid, 'Threw away %i probes (%2.1f%%) due to missing intensity\n', ...
              trash, 100*trash/length(idx));
    end
    
    obs_seq = signal(:,idx(idx_no_nan));
    true_label_seq = label(idx(idx_no_nan));
    % decode predicted label sequence
    pred_path = decode_Viterbi(obs_seq, transition_scores, score_plifs, PAR);
    pred_label_seq = pred_path.label_seq;
    chunk_probe_id = probe_id(idx(idx_no_nan));
    
    % parse predicted expression level from state sequence
    pred_level = states_to_levels(pred_path.state_seq, PAR);
    
    [tmp, kmers_exact_filter]   = intersect(chunk_probe_id, kmers_exact(1,:));
    [tmp, kmers_inexact_filter] = intersect(chunk_probe_id, kmers_inexact(1,:));
    [tmp, kmers_short_filter]   = intersect(chunk_probe_id, kmers_short(1,:));
    kmers_all_filter = unique([kmers_exact_filter, ...
                               kmers_inexact_filter, ...
                               kmers_short_filter]);
    if VERBOSE>=2,
      fprintf(out_fid, '    %3i ', c);
    end
    q = q + 1;
    predictions(q).array_fn = array_fn;
    predictions(q).chunk = c;
    predictions(q).probe_id = chunk_probe_id;
    predictions(q).true_label_seq = true_label_seq;
    predictions(q).pred_label_seq = pred_label_seq;
    predictions(q).pred_level = pred_level;
    predictions(q).predictor = p;
    assert(size(chunk_probe_id,2)==size(true_label_seq,2));
    assert(size(chunk_probe_id,2)==size(pred_label_seq,2));
      
    good_pos = find(true_label_seq~=LABELS.ambiguous ...
                    & true_label_seq~=LABELS.genic);
    predictions(q).acc_LSL = mean((true_label_seq(good_pos)==LABELS.exonic) == ...
                                  (pred_label_seq(1,good_pos)==LABELS.exonic) ...
                                  & (true_label_seq(good_pos)~=LABELS.exonic) == ...
                                  (pred_label_seq(1,good_pos)~=LABELS.exonic));
    mean_active_I(q) = mean(obs_seq(true_label_seq==LABELS.exonic));
    mean_inactive_I(q) = mean(obs_seq(true_label_seq==LABELS.intronic ...
                                      | true_label_seq==LABELS.intergenic));
    mean_active_R(q) = length(true_label_seq(kmers_all_filter)==LABELS.exonic) ... 
                       / sum(true_label_seq==LABELS.exonic);
    mean_inactive_R(q) = length(true_label_seq(kmers_all_filter)==LABELS.intronic ...
                                | true_label_seq(kmers_all_filter)==LABELS.intergenic) ...
                       / sum(true_label_seq==LABELS.intronic ...
                             | true_label_seq==LABELS.intergenic);
    frac_missing(q) = 1 - length(idx_no_nan)/length(idx);
  end
  if VERBOSE>=1,
    fprintf(out_fid, 'Average performance of predictor %i: %2.1f%%\n', p, ...
            100*mean([predictions(find([predictions.predictor]==p)).acc_LSL]));
  end
end

[tmp sort_idx] = sort([predictions.chunk]);
predictions     = predictions(sort_idx);
mean_active_I   = mean_active_I(sort_idx);
mean_inactive_I = mean_inactive_I(sort_idx);
mean_active_R   = mean_active_R(sort_idx);
mean_inactive_R = mean_inactive_R(sort_idx);
frac_missing    = frac_missing(sort_idx);

chunk_id     = [predictions.chunk];
probe_id     = [predictions.probe_id];
true_label   = [predictions.true_label_seq];
pred_label   = [predictions.pred_label_seq];
pred_level   = [predictions.pred_level];
predictor_id = [predictions.predictor];
pred_exons   = find_blocks(pred_label(1,:)==LABELS.exonic);
for e=1:size(pred_exons,2),
  pred_exons(3,e) = mean(pred_level(pred_exons(1,e):pred_exons(2,e))); 
end

% save predictions
save([base_dir 'prediction/genome_segmentation/' array_id '_' dt '_segmented.mat'], ...
     'array_fn', 'chunk_id', 'probe_id', 'predictor_id', ...
     'true_label', 'pred_label', 'pred_exons');

% print a figure for monitoring prediction performance along the genome
if VERBOSE>=1,
  figure;
  colors = [0 0.6 0.2;
            0 0.3 0.6];
  for c=1:5,
    subplot(5,1,c);
    hold on
    x = [];
    y_act = [];
    y_inact = [];
    r_act = [];
    r_inact = [];
    y_miss = [];
    acc = [];
    for i=1:length(predictions),
      if floor(predictions(i).probe_id(1)/10^8)==c,
        acc(end+1) = predictions(i).acc_LSL;
        plot([predictions(i).probe_id(1), predictions(i).probe_id(end)], ...
             [acc(end) acc(end)], ...
             'LineWidth', 2, 'Color', colors(predictions(i).predictor,:));
        
        x(end+1) = (predictions(i).probe_id(end) ...
                    + predictions(i).probe_id(1))/2;
        y_act(end+1) = mean_active_I(i);
        y_inact(end+1) = mean_inactive_I(i);
        r_act(end+1) = mean_active_R(i);
        r_inact(end+1) = mean_inactive_R(i);
        y_miss(end+1) = frac_missing(i);
      end
    end
    fprintf(out_fid, 'Mean accuracy on chr %i: %2.1f%%\n', c, 100*mean(acc));
    plot(x, y_act/mean(mean_inactive_I), 'r');
    plot(x, y_inact/mean(mean_inactive_I), 'k');
    plot(x, 1.2+0.2*r_act, 'r+');
    plot(x, 1.2+0.2*r_inact, 'k+');
    plot(x, y_miss, 'g+');
    array_id(find(array_id=='_')) = ' ';
    title(sprintf('chr %i (%s)', c, array_id));
    grid on
    axis([c*10^8 c*10^8+3.2*10^7 0 1.8]);
  end
  
  orient tall;
  array_id(find(array_id==' ')) = '_';
  print('-depsc', '-r300', ...
        [base_dir 'prediction/genome_segmentation/' array_id '_' dt '_performance.eps']);
end

if WRITE_SUMMARY_FILE,
  assert(fclose(out_fid)==0);
end

% eof