function [eval T1 T2 au_ROC] = eval_genes(genes, probe_label, LABEL, NUM_QUANTILES)

% [eval T1 T2 au_ROC] = eval_genes(genes, probe_label, LABEL, [NUM_QUANTILES])
%
% Evaluates the separation between exonic and intronic probes for
% different normalizations.
%
% genes -- a struct array of gene models
% probe_label -- a global annotation label for each probe
% LABEL -- a struct with all possible probe label constants
% NUM_QUANTILES -- optional parameter specifying the number of discrete
%   gene expression levels
% returns an evaluation struct with substructures for different
%   normalization methods, T1 and T2 structs for deviations from constant
%   trabscript intensities and the area under the ROC curve averaged fro
%   all genes
%
% written by Georg Zeller, MPI Tuebingen, Germany, 2007-2008

if ~exist('NUM_QUANTILES', 'var'),
  NUM_QUANTILES = 10;
end

eval = struct;
eval.RAW = struct;
eval.RAW.au_ROC = nan(1,length(genes));
if isfield(genes(1).hyb, 'I_n_SVR'),
  eval.SVR = struct;
  eval.SVR.au_ROC = nan(1,length(genes));
  eval.SVR.T1_exo = nan(1,length(genes));
  eval.SVR.T2_exo = nan(1,length(genes));
  eval.SVR.T1_ino = nan(1,length(genes));
  eval.SVR.T2_ino = nan(1,length(genes));
  eval.SVR.num_p  = zeros(1,length(genes));
end
if isfield(genes(1).hyb, 'I_n_RR'),
  eval.RR = struct;
  eval.RR.au_ROC = nan(1,length(genes));
  eval.RR.T1_exo = nan(1,length(genes));
  eval.RR.T2_exo = nan(1,length(genes));
  eval.RR.T1_ino = nan(1,length(genes));
  eval.RR.T2_ino = nan(1,length(genes));
  eval.RR.num_p  = zeros(1,length(genes));
end
if isfield(genes(1).hyb, 'I_n_SQN'),
  eval.SQN = struct;
  eval.SQN.au_ROC = nan(1,length(genes));
  eval.SQN.T1_exo = nan(1,length(genes));
  eval.SQN.T2_exo = nan(1,length(genes));
  eval.SQN.T1_ino = nan(1,length(genes));
  eval.SQN.T2_ino = nan(1,length(genes));
  eval.SQN.num_p  = zeros(1,length(genes));
end

eval.gene.num_transcripts = zeros(1,length(genes));
eval.gene.I_transcript    = nan(1,length(genes));
eval.gene.is_nan          = ones(1,length(genes));

for g=1:length(genes),
  gene = genes(g);
  assert(isfield(gene, 'hyb'));
  assert(isfield(gene, 'exons')), 

  eval.gene.num_transcripts(g) = length(gene.transcripts);
  eval.gene.I_transcript(g)    = mean(gene.hyb.I_transcript);

  label_gene = probe_label(gene.probes(:,2));
  is_ambiguous = label_gene == LABEL.ambiguous;
  is_nan = isnan(gene.hyb.I)';

  all_exonic = sum(gene.probes_exonic>0,2)==length(gene.transcripts);
%  assert(all(label_gene(all_exonic)~=LABEL.intronic & label_gene(all_exonic)~=LABEL.intergenic));
  all_intronic = sum(gene.probes_intronic>0,2)==length(gene.transcripts);
%  assert(all(label_gene(all_intronic)~=LABEL.exonic));
 
  if sum(all_exonic(~is_nan & ~is_ambiguous)) < 1,
%    fprintf('Skipping gene %s, because it has no exon intensities.\n', ...
%            gene.name);
    % gene will be skipped in evaluation later as gene.eval.is_nan == 1
    continue
  end
  if sum(all_intronic(~is_nan & ~is_ambiguous)) < 1,
%    fprintf('Skipping gene %s, because it has no intron intensities.\n', ...
%            gene.name);
    % gene will be skipped in evaluation later as gene.eval.is_nan == 1
    continue
  end

  legends = {'raw intensities'};
  I_raw = gene.hyb.I';
  I_gene = I_raw;
  
  exo_idx = find(all_exonic(~is_ambiguous));
  ino_idx = find(all_intronic(~is_ambiguous));
  if isfield(gene.hyb, 'I_n_SVR'),
    legends{end+1} = 'SVR intensities';
    I_n_SVR = gene.hyb.I_n_SVR';
    I_gene(end+1,:) = I_n_SVR;
    is_nan = is_nan | isnan(gene.hyb.I_n_SVR)';
    
    eval.SVR.T1_exo(g) = mean(abs(I_n_SVR(exo_idx) - median(I_n_SVR(exo_idx)))) ...
        / median(I_raw(exo_idx));
    eval.SVR.T2_exo(g) = sqrt(mean((I_n_SVR(exo_idx) - mean(I_n_SVR(exo_idx))).^2)) ...
        / mean(I_raw(exo_idx));
    eval.SVR.T1_ino(g) = mean(abs(I_n_SVR(ino_idx) - median(I_n_SVR(ino_idx)))) ...
        / median(I_raw(ino_idx));
    eval.SVR.T2_ino(g) = sqrt(mean((I_n_SVR(ino_idx) - mean(I_n_SVR(ino_idx))).^2)) ...
        / mean(I_raw(ino_idx));
  end
  if isfield(gene.hyb, 'I_n_RR'),
    legends{end+1} = 'RR intensities';
    I_n_RR = gene.hyb.I_n_RR';
    I_gene(end+1,:) = I_n_RR;
    is_nan = is_nan | isnan(gene.hyb.I_n_RR)';

    eval.RR.T1_exo(g) = mean(abs(I_n_RR(exo_idx) - median(I_n_RR(exo_idx)))) ...
        / median(I_raw(exo_idx));
    eval.RR.T2_exo(g) = sqrt(mean((I_n_RR(exo_idx) - mean(I_n_RR(exo_idx))).^2)) ...
        / mean(I_raw(exo_idx));
    eval.RR.T1_ino(g) = mean(abs(I_n_RR(ino_idx) - median(I_n_RR(ino_idx)))) ...
        / median(I_raw(ino_idx));
    eval.RR.T2_ino(g) = sqrt(mean((I_n_RR(ino_idx) - mean(I_n_RR(ino_idx))).^2)) ...
        / mean(I_raw(ino_idx));
  end
  if isfield(gene.hyb, 'I_n_SQN'),
    legends{end+1} = 'SQN intensities';
    I_n_SQN = gene.hyb.I_n_SQN';
    I_gene(end+1,:) = I_n_SQN;
    is_nan = is_nan | isnan(gene.hyb.I_n_SQN)';
    
    eval.SQN.T1_exo(g) = mean(abs(I_n_SQN(exo_idx) - median(I_n_SQN(exo_idx)))) ...
        / median(I_raw(exo_idx));
    eval.SQN.T2_exo(g) = sqrt(mean((I_n_SQN(exo_idx) - mean(I_n_SQN(exo_idx))).^2)) ...
        / mean(I_raw(exo_idx));
    eval.SQN.T1_ino(g) = mean(abs(I_n_SQN(ino_idx) - median(I_n_SQN(ino_idx)))) ...
        / median(I_raw(ino_idx));
    eval.SQN.T2_ino(g) = sqrt(mean((I_n_SQN(ino_idx) - mean(I_n_SQN(ino_idx))).^2)) ...
        / mean(I_raw(ino_idx));
  end
  
  eval.gene.is_nan(g) = mean(is_nan);
  
  for i=1:size(I_gene,1),
    if all(isnan(I_gene(i,:))), 
      continue
    end
    
    idx = find(~isnan(I_gene(i,:)));
    label = zeros(1,length(idx));
    label(exo_idx) = 1;
    label(ino_idx) = -1;
    assert(length(label) == length(idx));

    [TP FP TN FN] = eval_separation(I_gene(i,idx), label);
%    figure(1)
%    clf
%    hold on
%    plot(idx, I_gene(i,idx), 'k.-');
%    plot(idx, label, 'r.-');
%    axis([0 length(idx)+1 -2 16]);
%    pause
    
    % 1-Specificity
    % add a tiny bit to denominator to avoid division by 0
    FPR = FP./(FP+TN + 10^-6); 
    % Sensitivity (=Recall)
    % add a tiny bit to denominator to avoid division by 0
    TPR = TP./(TP+FN + 10^-6);
    
    % compute area under the curve
    FPR = FPR(end:-1:1);
    TPR = TPR(end:-1:1);
    assert(FPR(1)<0+10-5 & FPR(end)>1-10^-5);
    AUC = area_trapz(FPR, TPR);
    fn = fieldnames(eval);
    fn = fn{i};
    tmp = getfield(eval, fn);
    tmp.au_ROC(g) = AUC;
    eval = setfield(eval, fn, tmp);
  end
  if mod(g,100)==0, fprintf('  %i\r',g); end
end
fprintf('\n');

% calculate ROC scores for genes with less than 50% NaN intensities that
% have only one isoform annotated and ideal transcript intensity is not Nan
idx = find(eval.gene.is_nan<0.5 ...
           & eval.gene.num_transcripts==1 ...
           & ~isnan(eval.gene.I_transcript));
fprintf('Analyzing %.2f%% of test genes.\n', 100*length(idx)/length(genes))

q_idx = split_data([eval.gene.I_transcript(idx)], NUM_QUANTILES);
for i=1:NUM_QUANTILES,
  take_idx = idx(q_idx==i);
  
  tmp = eval.RAW.au_ROC(take_idx);
  au_ROC.RAW(i) = mean(tmp(~isnan(tmp)));

  if isfield(eval, 'SVR'),
    tmp = eval.SVR.T1_exo(take_idx);
    T1.SVR(i) = mean(tmp(~isnan(tmp)));
    tmp = eval.SVR.T2_exo(take_idx);
    T2.SVR(i) = mean(tmp(~isnan(tmp)));
    tmp = eval.SVR.au_ROC(take_idx);
    au_ROC.SVR(i) = mean(tmp(~isnan(tmp)));
  end
  
  if isfield(eval, 'RR'),
    tmp = eval.RR.T1_exo(take_idx);
    T1.RR(i) = mean(tmp(~isnan(tmp)));
    tmp = eval.RR.T2_exo(take_idx);
    T2.RR(i) = mean(tmp(~isnan(tmp)));
    tmp = eval.RR.au_ROC(take_idx);
    au_ROC.RR(i) = mean(tmp(~isnan(tmp)));
  end
  
  if isfield(eval, 'SQN'),
    tmp = eval.SQN.T1_exo(take_idx);
    T1.SQN(i) = mean(tmp(~isnan(tmp)));
    tmp = eval.SQN.T2_exo(take_idx);
    T2.SQN(i) = mean(tmp(~isnan(tmp)));
    tmp = eval.SQN.au_ROC(take_idx);
    au_ROC.SQN(i) = mean(tmp(~isnan(tmp)));
  end
end


if isfield(eval, 'SVR'),
  tmp = eval.SVR.T1_ino(idx);
  T1.SVR(end+1) = mean(tmp(~isnan(tmp)));
  tmp = eval.SVR.T2_ino(idx);
  T2.SVR(end+1) = mean(tmp(~isnan(tmp)));
end  
if isfield(eval, 'RR'),
  tmp = eval.RR.T1_ino(idx);
  T1.RR(end+1) = mean(tmp(~isnan(tmp)));
  tmp = eval.RR.T2_ino(idx);
  T2.RR(end+1) = mean(tmp(~isnan(tmp)));
end
if isfield(eval, 'SQN'),
  tmp = eval.SQN.T1_ino(idx);
  T1.SQN(end+1) = mean(tmp(~isnan(tmp)));
  tmp = eval.SQN.T2_ino(idx);
  T2.SQN(end+1) = mean(tmp(~isnan(tmp)));
end

